package demo.miscellaneous;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.util.List ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.draw.ChangeChartShapeAdapter ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryStepChartDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public CategoryStepChartDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryStepChart("Category Step Chart", "Category",
                                                               "Value", dataset, orientation, true, true) ;
    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;
    plot.setDomainGridlinePaint(Color.white) ;
    plot.setValueGridlinePaint(Color.white) ;

    CategoryAxis categoryaxis = plot.getCategoryAxis() ;
    categoryaxis.setTickLabelAngle(IAngle.ANGLE_UP_45) ;
    categoryaxis.setLowerMargin(0.0D) ;
    categoryaxis.setUpperMargin(0.0D) ;

    chart.setChangeChartShape(new ChangeChartShapeAdapter() {
      public void beforeDraw(ChartShapeCollection chartShapes) {
        AxisShapeCollection axisShapes = chart.getChartShapes().getPlotShapes().getXAxisShape() ;
        List tickShapes = axisShapes.getTickLabelShapes() ;

        IChartShape shape = (IChartShape) tickShapes.get(0) ;
        shape.setTooltipText("The first type.") ;
        shape = (IChartShape) tickShapes.get(1) ;
        shape.setTooltipText("The second type.") ;
        shape = (IChartShape) tickShapes.get(2) ;
        shape.setTooltipText("The third type.") ;
      }

    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    for (int i = 1 ; i <= 3 ; i++) {
      dataset.addRowKey("Series " + i) ;
    }
    for (int i = 1 ; i <= 8 ; i++) {
      dataset.addColKey("Type " + i) ;
    }

    double data[][] = { {1.4, 4D, 3D, 5D, 2.5, 7.2D, 7.5D, 8D}
                      , {5.3D, 7.1D, 6.8D, 8.2D, 4.3D, 4.5D, 2.3D, 1.5D}
                      , {4.9D, 3.8D, 2.6D, 3.3D, 6.2D, 3.3D, 4.8D, 3.5D}
    } ;

    for (int i = 0 ; i < data.length ; i++) {
      dataset.addRowData() ;
      for (int j = 0 ; j < data[i].length ; j++) {
        dataset.addData(i, new NumberChartData(data[i][j])) ;
      }
    }

    return dataset ;
  }

}
