package demo.miscellaneous.crosshairs ;

import java.awt.BorderLayout ;
import java.awt.geom.Point2D ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;

import javax.swing.JPanel ;
import javax.swing.JScrollPane ;
import javax.swing.JTable ;
import javax.swing.table.AbstractTableModel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.event.ChartProgressEvent ;
import com.ijchart.xychart.listener.IChartProgressListener ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.shape.IChartShape ;
import demo.others.common.renderer.DateCellRenderer ;
import demo.others.common.renderer.NumberCellRenderer ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CrosshairDemo2 extends JPanel {

  private DefaultMatrixDataset[] datasetArr ;

  public CrosshairDemo2() {
    setLayout(new BorderLayout()) ;
    add(new DemoPanel()) ;
  }

  static class DemoTableModel extends AbstractTableModel {

    public int getColumnCount() {
      return 7 ;
    }

    public int getRowCount() {
      return data.length ;
    }

    public Object getValueAt(int i, int j) {
      return data[i][j] ;
    }

    public void setValueAt(Object obj, int i, int j) {
      data[i][j] = obj ;
      fireTableDataChanged() ;
    }

    public String getColumnName(int i) {
      switch (i) {
      case 0: // '\0'
        return "Series Name:" ;

      case 1: // '\001'
        return "X:" ;

      case 2: // '\002'
        return "Y:" ;

      case 3: // '\003'
        return "X (prev)" ;

      case 4: // '\004'
        return "Y (prev):" ;

      case 5: // '\005'
        return "X (next):" ;

      case 6: // '\006'
        return "Y (next):" ;
      }
      return null ;
    }

    private Object data[][] ;

    public DemoTableModel(int i) {
      data = new Object[i][7] ;
    }
  }

  private class DemoPanel extends JPanel implements IChartProgressListener {
    private ChartPanel chartPanel ;

    private DemoTableModel model ;

    private IJChart chart ;

    private IJChart createChart() {
      datasetArr = new DefaultMatrixDataset[4] ;
      for (int i = 0 ; i < 4 ; i++) {
        datasetArr[i] = createDataset("Series " + i, 100D + (double) i * 200D, 200) ;
      }

      chart = ChartFactory.createXYLineAndShapeChart("Crosshair Demo 2", "Time of Day", "Value",
                                                     datasetArr[0], ChartOrientation.VERTICAL, true, false,
                                                     true, false) ;
      XYPlot plot = (XYPlot) chart.getPlot() ;
      plot.setDomainCrosshairVisible(true) ;
      plot.setValueCrosshairVisible(true) ;

      for (int i = 1 ; i < 4 ; i++) {
        XYNumberValueAxis valueAxis = new XYNumberValueAxis("Axis " + (i + 1), datasetArr[i],
                                                            XYValueAxisType.XY_VALUE_AXIS_Y) ;
        valueAxis.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
        plot.setValueAxis(i, valueAxis) ;

        XYLineAndShapeRender render = new XYLineAndShapeRender(datasetArr[i]) ;
        render.setBaseItemShapeVisible(false) ;
        plot.setRender(i, render) ;
        plot.mapRenderToValueAxis(i, i) ;
        plot.mapRenderToDomainAxis(i, 0) ;
      }

      XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
      render.setBaseItemShapeVisible(false) ;

      return chart ;
    }

    private DefaultMatrixDataset createDataset(String s, double d, int j) {
      DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
      dataset.addRowKey(s) ;
      dataset.addRowData() ;

      Calendar cal = Calendar.getInstance() ;
      cal.set(Calendar.SECOND, 0) ;
      cal.set(Calendar.MILLISECOND, 0) ;

      double d1 = d ;
      for (int k = 0 ; k < j ; k++) {
        dataset.addData(0, new XYNumberChartData(cal.getTimeInMillis(), d1)) ;
        d1 *= 1.0D + (Math.random() - 0.495D) / 10D ;

        cal.add(Calendar.MINUTE, 1) ;
      }

      return dataset ;
    }

    public void chartProgress(ChartProgressEvent chartprogressevent) {
      if (chartprogressevent.getType() != ChartProgressEvent.PROGRESS_DRAWING_SHAPES_FINISHED) {
        return ;
      }

      for (int i = 0 ; i < 4 ; i++) {
        model.setValueAt(datasetArr[i].getRowKey(0), i, 0) ;

        model.setValueAt(null, i, 1) ;
        model.setValueAt(null, i, 2) ;
        model.setValueAt(null, i, 3) ;
        model.setValueAt(null, i, 4) ;
        model.setValueAt(null, i, 5) ;
        model.setValueAt(null, i, 6) ;
      }

      XYPlot xyplot = (XYPlot) chart.getPlot() ;

      Point2D crossPoint = xyplot.getCrosshairPoint(chartPanel.getMouseClickPoint()) ;
      if (crossPoint == null) {
        return ;
      }

      IChartShape shape = chartPanel.getShapeForPoint(crossPoint,
                                                      ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;
      if (shape == null) {
        return ;
      }

      XYNumberChartData data = (XYNumberChartData) shape.getData() ;
      if (data != null) {
        int dataIndex = -1 ;
        for (int m = 0 ; m < 4 ; m++) {
          int colCount = datasetArr[m].getColumnCount(0) ;
          for (int i = 0 ; i < colCount ; i++) {
            if (data == datasetArr[m].getData(0, i)) {
              dataIndex = i ;
              break ;
            }
          }
        }
        if (dataIndex < 0) {
          return ;
        }

        for (int i = 0 ; i < 4 ; i++) {
          int colCount = datasetArr[i].getColumnCount(0) ;
          XYNumberChartData currData = (XYNumberChartData) datasetArr[i].
                                       getData(0, dataIndex) ;
          XYNumberChartData previousData = (XYNumberChartData) datasetArr[i].
                                           getData(0, Math.max(0, dataIndex - 1)) ;
          XYNumberChartData nextData = (XYNumberChartData) datasetArr[i].
                                       getData(0, Math.min(colCount - 1, dataIndex + 1)) ;
          if (previousData != null) {
            model.setValueAt(previousData.getXData(), i, 3) ;
            model.setValueAt(previousData.getYData(), i, 4) ;
          }

          model.setValueAt(currData.getXData(), i, 1) ;
          model.setValueAt(currData.getYData(), i, 2) ;

          if (nextData != null) {
            model.setValueAt(nextData.getXData(), i, 5) ;
            model.setValueAt(nextData.getYData(), i, 6) ;
          }
        }
      }

    }

    public DemoPanel() {
      setLayout(new BorderLayout()) ;
      chart = createChart() ;
      chart.addProgressListener(this) ;
      chartPanel = new ChartPanel(chart) ;
      add(chartPanel, BorderLayout.NORTH) ;

      model = new DemoTableModel(4) ;
      for (int i = 0 ; i < 4 ; i++) {
        model.setValueAt(datasetArr[i].getRowKey(0), 0, 0) ;
        model.setValueAt(new Double("0.00"), i, 1) ;
        model.setValueAt(new Double("0.00"), i, 2) ;
        model.setValueAt(new Double("0.00"), i, 3) ;
        model.setValueAt(new Double("0.00"), i, 4) ;
        model.setValueAt(new Double("0.00"), i, 5) ;
        model.setValueAt(new Double("0.00"), i, 6) ;
      }

      JTable jtable = new JTable(model) ;
      DateCellRenderer datecellrenderer = new DateCellRenderer(new SimpleDateFormat("HH:mm:ss")) ;
      NumberCellRenderer numbercellrenderer = new NumberCellRenderer() ;
      jtable.getColumnModel().getColumn(1).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(2).setCellRenderer(numbercellrenderer) ;
      jtable.getColumnModel().getColumn(3).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(4).setCellRenderer(numbercellrenderer) ;
      jtable.getColumnModel().getColumn(5).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(6).setCellRenderer(numbercellrenderer) ;

      JPanel panel1 = new JPanel(new BorderLayout()) ;
      panel1.add(new JScrollPane(jtable)) ;
      add(panel1, BorderLayout.CENTER) ;
    }

  }

}
