package demo.miscellaneous.dynamiccharts ;

import java.awt.BorderLayout ;
import java.awt.event.ActionEvent ;
import java.awt.event.ActionListener ;

import javax.swing.JButton ;
import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DynamicDataDemo2 extends JPanel {
  private ChartOrientation vertical = ChartOrientation.VERTICAL ;

  private double lastValue1 = 100 ;

  private double lastValue2 = 500 ;

  public DynamicDataDemo2() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    final DefaultMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createXYLineAndShapeChart("Dynamic Data Demo2", "Time",
                                                                 "Value", dataset, vertical, true, true, true, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setRowItemShapeVisible(0, false) ;
    render.setRowItemShapeVisible(1, true) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;

    JPanel buttonPanel = new JPanel() ;
    JButton button = new JButton("Add To Series 1") ;
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dataset.addData(0, createData1()) ;
      }

      private XYNumberChartData createData1() {
        double d = 0.90000000000000002D + 0.20000000000000001D * Math.random() ;
        lastValue1 *= d ;

        return new XYNumberChartData(System.currentTimeMillis(), lastValue1) ;
      }

    }) ;
    buttonPanel.add(button) ;

    button = new JButton("Add To Series 2") ;
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dataset.addData(1, createData2()) ;
      }

      private XYNumberChartData createData2() {
        double d = 0.90000000000000002D + 0.20000000000000001D * Math.random() ;
        lastValue2 *= d ;

        return new XYNumberChartData(System.currentTimeMillis(), lastValue2) ;
      }

    }) ;
    buttonPanel.add(button) ;

    button = new JButton("Add To Both") ;
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dataset.addData(0, createData1()) ;
        dataset.addData(1, createData2()) ;
      }

      private XYNumberChartData createData1() {
        double d = 0.90000000000000002D + 0.20000000000000001D * Math.random() ;
        lastValue1 *= d ;

        return new XYNumberChartData(System.currentTimeMillis(), lastValue1) ;
      }

      private XYNumberChartData createData2() {
        double d = 0.90000000000000002D + 0.20000000000000001D * Math.random() ;
        lastValue2 *= d ;

        return new XYNumberChartData(System.currentTimeMillis(), lastValue2) ;
      }

    }) ;
    buttonPanel.add(button) ;

    add(buttonPanel, BorderLayout.SOUTH) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Random 1") ;
    dataset.addRowKey("Random 2") ;
    dataset.addRowData(2) ;

    return dataset ;
  }

}
