package demo.miscellaneous.itemlabel ;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.text.NumberFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.LabelGeneratorAdaptor ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.category.AbstractCategoryRender ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ItemLabelDemo2 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  public ItemLabelDemo2() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    DefaultMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createCategoryBar2DChart("Item Label Demo 2", "Category",
                                                          "Value", dataset, orientation, true, true) ;
    chart.getLegendTitle().setPosition(RectangleEdge.RIGHT) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    AbstractCategoryRender render = plot.getRender() ;
    render.setBaseItemLabelVisible(true) ;
    render.setLabelGenerator(new LabelGenerator()) ;
    render.setBaseItemLabelPosition(TextPosition.OUTSIDE_RIGHT) ;
    ((NumberValueAxis) plot.getValueAxis()).setUserDefineDatasetRange(new NumberRange(0, 120)) ;

    plot.setValueGridlinePaint(Color.white) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("S1") ;
    dataset.addColKey("S2") ;
    dataset.addRowKey("C1") ;
    dataset.addRowKey("C2") ;
    dataset.addRowKey("C3") ;
    dataset.addRowData(3) ;

    dataset.addData(0, new NumberChartData(100)) ;
    dataset.addData(0, new NumberChartData(80)) ;
    dataset.addData(1, new NumberChartData(44.299999999999997D)) ;
    dataset.addData(1, new NumberChartData(75.099999999999994D)) ;
    dataset.addData(2, new NumberChartData(93D)) ;
    dataset.addData(2, new NumberChartData(15.1D)) ;

    return dataset ;
  }

  class LabelGenerator extends LabelGeneratorAdaptor {
    private NumberFormat formatter ;

    public LabelGenerator() {
      formatter = NumberFormat.getPercentInstance() ;
    }

    public String generateLabel(IMatrixDataset dataset, int row, int column) {
      String s = null ;
      NumberChartData number = (NumberChartData) dataset.getData(row, column) ;
      if (number != null) {
        double d = number.getValue() ;
        double colTotal = calculateColumnTotal(dataset, column) ;
        s = d + " (" + formatter.format(d / colTotal) + ")" ; ;
      }

      return s ;
    }

    private double calculateColumnTotal(IMatrixDataset dataset, int column) {
      int rows = dataset.getRowCount() ;
      double d = 0.0D ;
      for (int j = 0 ; j < rows ; j++) {
        NumberChartData number = (NumberChartData) dataset.getData(j, column) ;
        if (number != null) {
          d += number.getValue() ;
        }
      }

      return d ;
    }
  }
}
