package demo.statisticalcharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.BoxAndWhiskerValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.data.IntervalXBoxAndWhiskerData ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYBoxAndWhiskerRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BoxAndWhiskerChartDemo2 extends JPanel {

  private Calendar cal = Calendar.getInstance() ;

  private ChartOrientation vertical = ChartOrientation.VERTICAL ;

  public BoxAndWhiskerChartDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    SingleRowDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYBoxAndWhiskerChart(
      "Box-and-Whisker Chart Demo 2", "Day", "Value", dataset, vertical, true, true, true, false) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    BoxAndWhiskerValueAxis domainAxis = (BoxAndWhiskerValueAxis) plot.getDomainAxis() ;
    domainAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_DAY)) ;

    BoxAndWhiskerValueAxis valueAxis = (BoxAndWhiskerValueAxis) plot.getValueAxis() ;
    valueAxis.setTickUnit(new LongTickUnit()) ;

    XYBoxAndWhiskerRender render = (XYBoxAndWhiskerRender) plot.getRender() ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaint(Color.green) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private SingleRowDataset createDataset() {
    SingleRowDataset dataset = new SingleRowDataset() ;
    dataset.addRowKey("Series Name") ;

    cal.set(Calendar.HOUR_OF_DAY, 0) ;
    cal.set(Calendar.MINUTE, 0) ;
    cal.set(Calendar.SECOND, 0) ;
    cal.set(Calendar.MILLISECOND, 0) ;

    dataset.addRowData() ;
    for (int i = 0 ; i < 10 ; i++) {
      dataset.addData(0, createData()) ;
      cal.add(Calendar.DAY_OF_MONTH, 1) ;
    }

    return dataset ;
  }

  private IntervalXBoxAndWhiskerData createData() {
    final long offset = 6 * 60 * 60 * 1000 ;
    long start = cal.getTimeInMillis() - offset ;
    long end = cal.getTimeInMillis() + offset ;
    IntervalXBoxAndWhiskerData data = new IntervalXBoxAndWhiskerData(start, end) ;
    Double[] dataArr = createValueList(0.0D, 20D, 20) ;
    data.addValues(dataArr) ;

    return data ;
  }

  private static Double[] createValueList(double d, double d1, int i) {
    Double[] arraylist = new Double[i] ;
    for (int j = 0 ; j < i ; j++) {
      double d2 = d + Math.random() * (d1 - d) ;
      arraylist[j] = new Double(d2) ;
    }

    return arraylist ;
  }

}
