package demo.statisticalcharts ;

import java.awt.Color ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;
import javax.swing.JTabbedPane ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.function.LineFunction ;
import com.ijchart.xychart.function.PowerFunction ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.util.DatasetUtil ;
import com.ijchart.xychart.util.RegressionUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class RegressionDemo1 extends JPanel {
  private IMatrixDataset dataset = createDataset() ;

  public RegressionDemo1() {
    setLayout(new GridLayout(1, 1)) ;

    JTabbedPane jtabbedpane = new JTabbedPane() ;
    jtabbedpane.add("Linear", addPanel1()) ;
    jtabbedpane.add("Power", addPanel2()) ;
    add(jtabbedpane) ;
  }

  private ChartPanel addPanel1() {
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Linear Regression", "X", "Y",
                                                           dataset, ChartOrientation.VERTICAL, true,
                                                           true, false, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setBaseItemShapeVisible(true) ;
    render.setBaseItemLineVisible(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setLowerMargin(0.03) ;
    xAxis.setUpperMargin(0.03) ;
    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setTickUnit(new LongTickUnit()) ;

    double ad[] = RegressionUtil.getOLSRegression(dataset, 0) ;

    LineFunction lineFunction = new LineFunction(ad[0], ad[1]) ;
    XYNumberChartData[] datas = DatasetUtil.getDatasByFunction(lineFunction, 2D, 11D, 100) ;

    DefaultMatrixDataset dataset2 = new DefaultMatrixDataset() ;
    dataset2.addRowKey("Fitted Regression Line") ;
    dataset2.addRowData() ;
    dataset2.addDatas(0, datas) ;

    XYLineAndShapeRender render1 = new XYLineAndShapeRender(dataset2) ;
    render1.setRowItemFillPaint(0, Color.blue) ;
    render1.setBaseItemShapeVisible(false) ;
    render1.setBaseItemLineVisible(true) ;
    plot.setRender(1, render1) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    return chartpanel ;
  }

  private ChartPanel addPanel2() {
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Power Regression", "X", "Y",
                                                           dataset, ChartOrientation.VERTICAL, true,
                                                           true, false, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setBaseItemShapeVisible(true) ;
    render.setBaseItemLineVisible(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setLowerMargin(0.03) ;
    xAxis.setUpperMargin(0.03) ;
    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setTickUnit(new LongTickUnit()) ;

    double ad[] = RegressionUtil.getPowerRegression(dataset, 0) ;
    PowerFunction lineFunction = new PowerFunction(ad[0], ad[1]) ;
    XYNumberChartData[] datas = DatasetUtil.getDatasByFunction(lineFunction, 2D, 11D, 100) ;

    DefaultMatrixDataset dataset2 = new DefaultMatrixDataset() ;
    dataset2.addRowKey("Fitted Regression Line") ;
    dataset2.addRowData() ;
    dataset2.addDatas(0, datas) ;

    XYLineAndShapeRender render1 = new XYLineAndShapeRender(dataset2) ;
    render1.setRowItemFillPaint(0, Color.blue) ;
    render1.setBaseItemShapeVisible(false) ;
    render1.setBaseItemLineVisible(true) ;
    plot.setRender(1, render1) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    return chartpanel ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset localDataset = new DefaultMatrixDataset() ;
    localDataset.addRowKey("Series 1") ;
    localDataset.addRowData() ;

    localDataset.addData(0, new XYNumberChartData(2D, 56.270000000000003D)) ;
    localDataset.addData(0, new XYNumberChartData(3D, 41.32D)) ;
    localDataset.addData(0, new XYNumberChartData(4D, 31.449999999999999D)) ;
    localDataset.addData(0, new XYNumberChartData(5D, 30.050000000000001D)) ;
    localDataset.addData(0, new XYNumberChartData(6D, 24.690000000000001D)) ;
    localDataset.addData(0, new XYNumberChartData(7D, 19.780000000000001D)) ;
    localDataset.addData(0, new XYNumberChartData(8D, 20.940000000000001D)) ;
    localDataset.addData(0, new XYNumberChartData(9D, 16.73D)) ;
    localDataset.addData(0, new XYNumberChartData(10D, 14.210000000000001D)) ;
    localDataset.addData(0, new XYNumberChartData(11D, 12.44D)) ;

    return localDataset ;
  }

}
