package demo.timeseriescharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.client.IJChartInfo ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TimeSeriesDemo4 extends JPanel {
  private Calendar cal = Calendar.getInstance() ;

  public TimeSeriesDemo4() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Time Series Demo 4", "Time",
                                                           "Value", dataset,
                                                           ChartOrientation.VERTICAL,
                                                           true, true, true, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    XYLineAndShapeRender render = (XYLineAndShapeRender) xyplot.getRender() ;
    render.setBaseItemShapeVisible(false) ;
    render.setRowItemFillPaint(0, Color.red) ;
    render.setRowItemFill(1, false) ;
    render.setToolTipGenerator(new DefaultTooltipGenerator()) ;
    render.setBackgroundImage(IJChartInfo.getLogoImage()) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_HOUR)) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowData() ;

    dataset.addData(0, createData(1, 500.19999999999999D)) ;
    dataset.addData(0, createData(2, 694.10000000000002D)) ;
    dataset.addData(0, createData(3, 734.39999999999998D)) ;
    dataset.addData(0, createData(4, 453.19999999999999D)) ;
    dataset.addData(0, createData(7, 500.19999999999999D)) ;
    dataset.addData(0, createData(8, Double.NaN)) ;
    dataset.addData(0, createData(12, 734.39999999999998D)) ;
    dataset.addData(0, createData(16, 453.19999999999999D)) ;

    return dataset ;
  }

  private XYNumberChartData createData(int hour, double value) {
    int year = cal.get(Calendar.YEAR) ;
    int month = cal.get(Calendar.MONTH) + 1 ;
    int day = cal.get(Calendar.DAY_OF_MONTH) ;

    return new XYNumberChartData(DateUtil.dateToTimeInMillis(year, month, day, hour), value) ;
  }

}
