package demo.xycharts ;

import java.awt.Color ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.XYZNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.paint.GrayPaintScale ;
import com.ijchart.xychart.render.xy.XYBlockRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.LegendTitle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBlockChartDemo1 extends JPanel {

  public XYBlockChartDemo1() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYBlockChart(
      "XYBlockChartDemo1", "X", "Y", "Scale", dataset, ChartOrientation.VERTICAL,
      true, true, false, false) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    plot.setBorderPaint(Color.BLACK);
    LegendTitle legendTitle = chart.getLegendTitle() ;
    legendTitle.setPosition(RectangleEdge.LEFT) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setAxisLinePaint(Color.white) ;
    xAxis.setTickMarkPaint(Color.white) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setTickUnit(new LongTickUnit()) ;
    yAxis.setAxisLinePaint(Color.white) ;
    yAxis.setTickMarkPaint(Color.white) ;

    XYBlockRender render = (XYBlockRender) plot.getRender() ;
    GrayPaintScale graypaintscale = new GrayPaintScale( -2D, 1.0D) ;
    render.setPaintScale(graypaintscale) ;

    plot.setDomainGridlineVisible(false) ;
    plot.setValueGridlinePaint(Color.white) ;
    plot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;

    legendTitle.setPadding(new RectangleInsets(5D, 5D, 5D, 5D)) ;
    legendTitle.setBorderPaint(Color.red) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    return new DefaultMatrixDataset() {

      public int getRowCount() {
        return 1 ;
      }

      public int getColumnCount() {
        return 10000 ;
      }

      public IChartData getData(int i, int j) {
        double x = j / 100 - 50 ;
        double y = j - (j / 100) * 100 - 50 ;
        double z = Math.sin(Math.sqrt(x * x + y * y) / 5D) ;

        return new XYZNumberChartData(x, y, z) ;
      }

      public Object getRowKey(int i) {
        return "sin(sqrt(x + y))" ;
      }

    } ;

  }

}
