package com.ijchart.xychart.annotation ;

import java.awt.Color ;
import java.awt.Font ;
import java.awt.Paint ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.TextAnchor ;
import com.ijchart.xychart.constant.TextDirection ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>ı͵ע(annotation)Ļ.</zh>
 * <en>A base class for text annotations. </en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractTextAnnotation extends AbstractAnnotation implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3203881447311572840L ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>The default font.</en>
   */
  public static final Font DEFAULT_FONT = new Font("SansSerif", Font.PLAIN, 10) ;

  /**
   * <zh>ȱʡıɫ.</zh>
   * <en>The default paint.</en>
   */
  public static final Paint DEFAULT_PAINT = Color.black ;

  /**
   * <zh>ȱʡתĽǶ.</zh>
   * <en>The default rotation angle.</en>
   */
  public static final double DEFAULT_ROTATION_ANGLE = 0.0 ;

  /**
   * <zh>ȱʡˮƽ뷽ʽ.</zh>
   * <en>The default text horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_TEXT_HORIZONTAL_ALIGNMENT =
    HorizontalAlignment.LEFT ;

  /**
   * <zh>ȱʡֱִ뷽ʽ.</zh>
   * <en>The default text vertical alignment.</en>
   */
  public static final VerticalAlignment DEFAULT_TEXT_VERTICAL_ALIGNMENT =
    VerticalAlignment.CENTER ;

  /**
   * <zh>ȱʡֺͼελùϵ.</zh>
   * <en>The default text position.</en>
   */
  public static final TextPosition DEFAULT_TEXT_POSITION = TextPosition.INSIDE ;

  /**
   * <zh>ȱʡַ.</zh>
   * <en>The default text direction.</en>
   */
  public static final TextDirection DEFAULT_TEXT_DIRECTION = TextDirection.HORIZONTAL ;

  /**
   * <zh>ҪƵı.</zh>
   * <en>The text.</en>
   */
  private String text ;

  /**
   * <zh>.</zh>
   * <en>The font.</en>
   */
  private Font textFont ;

  /**
   * <zh>ıɫ.</zh>
   * <en>The paint.</en>
   */
  private transient Paint textPaint ;

  /**
   * <zh>ֶ뷽ʽ.</zh>
   * <en>The position of a text.</en>
   */
  private TextAnchor textAnchor ;

  /**
   * <zh>ıתĽǶ.</zh>
   * <en>The rotation angle.</en>
   */
  private double rotationAngle ;

  /**
   * <zh>ˮƽ뷽ʽ.</zh>
   * <en>The text horizontal align.</en>
   */
  private HorizontalAlignment textHorizontalAlign ;

  /**
   * <zh>ֱִ뷽ʽ.</zh>
   * <en>The text vertical align.</en>
   */
  private VerticalAlignment textVerticalAlign ;

  /**
   * <zh>ıλ</zh>
   * <en>The text position.</en>
   */
  private TextPosition textPosition ;

  /**
   * <zh>ı.</zh>
   * <en>The text direction.</en>
   */
  private TextDirection textDirection ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param text <zh>ҪƵı (Ϊnull).</zh>
   * <en>The text (null not permitted).</en>
   */
  protected AbstractTextAnnotation(String text) {
    if (text == null) {
      throw new IllegalArgumentException("Null 'text' argument.") ;
    }

    this.text = text ;
    this.textFont = DEFAULT_FONT ;
    this.textPaint = DEFAULT_PAINT ;
    this.rotationAngle = DEFAULT_ROTATION_ANGLE ;

    this.textHorizontalAlign = DEFAULT_TEXT_HORIZONTAL_ALIGNMENT ;
    this.textVerticalAlign = DEFAULT_TEXT_VERTICAL_ALIGNMENT ;
    this.textPosition = DEFAULT_TEXT_POSITION ;
    this.textDirection = DEFAULT_TEXT_DIRECTION ;
  }

  /**
   * <zh>ʾ.</zh>
   * <en>Returns the text.</en>
   *
   * @return String <zh>ʾ (Ϊnull).</zh>
   * <en>The text (maybe null).</en>
   */
  public String getText() {
    return this.text ;
  }

  /**
   * <zh>ʾ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the text and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null not permitted).</en>
   */
  public void setText(String text) {
    if (text == null) {
      throw new IllegalArgumentException("Null 'text' argument.") ;
    }

    if (!EqualsUtil.equals(this.text, text)) {
      this.text = text ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Returns the text font.</en>
   *
   * @return Font <zh>.</zh>
   * <en>The text font.</en>
   */
  public Font getTextFont() {
    return this.textFont ;
  }

  /**
   * <zh>ֵ,
   * м{@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the text font and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh>ֵ (Ϊnull).</zh>
   * <en>The text font (null not permitted).</en>
   */
  public void setTextFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    this.textFont = font ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>Paint.</zh>
   * <en>Get the text Paint.</en>
   *
   * @return Paint <zh>Paint.</zh>
   * <en>The text Paint.</en>
   */
  public Paint getTextPaint() {
    return this.textPaint ;
  }

  /**
   * <zh>ֵPaint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the text Paint and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint(Ϊnull)</zh>
   * <en>The text Paint(null not permitted).</en>
   */
  public void setTextPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.textPaint = paint ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>תĽǶ(λǻ).</zh>
   * <en>Returns the rotation angle in radians.</en>
   *
   * @return <zh>Ƕ.</zh>
   * <en>The rotation angle.</en>
   */
  public double getRotationAngle() {
    return this.rotationAngle ;
  }

  /**
   * <zh>תĽǶ(λǻ),м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the rotation angle in radians and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param angle <zh>Ƕ.</zh>
   * <en>The angle (in radians).</en>
   */
  public void setRotationAngle(double angle) {
    if (this.rotationAngle != angle) {
      this.rotationAngle = angle ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ֵˮƽ뷽ʽ.</zh>
   * <en>Returns the text horizontal alignment.</en>
   *
   * @return HorizontalAlignment <zh>ˮƽ뷽ʽ.</zh>
   * <en>The text horizontal alignment.</en>
   */
  public HorizontalAlignment getTextHorizontalAlign() {
    return this.textHorizontalAlign ;
  }

  /**
   * <zh>ˮƽ뷽ʽ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the text horizontal alignment and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param horizontalAlign HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The text horizontal alignment (null not permitted).</en>
   */
  public void setTextHorizontalAlign(HorizontalAlignment horizontalAlign) {
    if (horizontalAlign == null) {
      throw new IllegalArgumentException("Null 'horizontalAlign' argument.") ;
    }

    if (!this.textHorizontalAlign.equals(horizontalAlign)) {
      this.textHorizontalAlign = horizontalAlign ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ֵĴֱ뷽ʽ.</zh>
   * <en>Returns the text vertical alignment.</en>
   *
   * @return VerticalAlignment <zh>ֱִ뷽ʽ.</zh>
   * <en>The text vertical alignment.</en>
   */
  public VerticalAlignment getTextVerticalAlign() {
    return this.textVerticalAlign ;
  }

  /**
   * <zh>ֱִ뷽ʽ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the text vertical alignment and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param verticalAlign VerticalAlignment <zh>ֱִ뷽ʽ (Ϊnull).</zh>
   * <en>The text vertical alignment (null not permitted).</en>
   */
  public void setTextVerticalAlign(VerticalAlignment verticalAlign) {
    if (verticalAlign == null) {
      throw new IllegalArgumentException("Null 'verticalAlign' argument.") ;
    }

    if (!this.textVerticalAlign.equals(verticalAlign)) {
      this.textVerticalAlign = verticalAlign ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ֺͼελùϵ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the text position and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param position TextPosition <zh>ֺͼελùϵ (Ϊnull).</zh>
   * <en>The text position (null not permitted).</en>
   */
  public void setTextPosition(TextPosition position) {
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument.") ;
    }

    if (!this.textPosition.equals(position)) {
      this.textPosition = position ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ֺͼελùϵ.</zh>
   * <en>Returns the text position.</en>
   *
   * @return TextPosition <zh>ֺͼελùϵ.</zh>
   * <en>The text position.</en>
   */
  public TextPosition getTextPosition() {
    return this.textPosition ;
  }

  /**
   * <zh>ֵķ.</zh>
   * <en>Returns the text direction.</en>
   *
   * @return TextDirection <zh>ֵķ (Ϊnull).</zh>
   * <en>The labe direction (never null).</en>
   */
  public TextDirection getTextDirection() {
    return this.textDirection ;
  }

  /**
   * <zh>ֵķ,
   * м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the text direction and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param direction TextDirection <zh>ֵķ (Ϊnull).</zh>
   * <en>The text direction (null not permitted).</en>
   */
  public void setTextDirection(TextDirection direction) {
    if (direction == null) {
      throw new IllegalArgumentException("Null 'direction' argument.") ;
    }

    if (!this.textDirection.equals(direction)) {
      this.textDirection = direction ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ˮƽ뷽ʽתΪTextAnchor.</zh>
   * <en>Convert horizontal to anchor.</en>
   *
   * @param align HorizontalAlignment
   * @return TextAnchor
   */
  protected TextAnchor horizontalAlignToAnchor(HorizontalAlignment align) {
    if (align == null) {
      return null ;
    } else if (getTextHorizontalAlign().equals(HorizontalAlignment.LEFT)) {
      return TextAnchor.START ;
    } else if (getTextHorizontalAlign().equals(HorizontalAlignment.CENTER)) {
      return TextAnchor.MIDDLE ;
    } else if (getTextHorizontalAlign().equals(HorizontalAlignment.RIGHT)) {
      return TextAnchor.END ;
    }

    return null ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.text) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textAnchor) ;
    result = 37 * result + this.rotationAngle ;
    result = 37 * result + HashcodeUtil.hashCode(this.textHorizontalAlign) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textVerticalAlign) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textPosition) ;
    result = 37 * result + HashcodeUtil.hashCode(this.textDirection) ;

    return (int) result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractTextAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractTextAnnotation that = (AbstractTextAnnotation) obj ;
    if (!EqualsUtil.equals(this.text, that.text)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textFont, that.textFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textPaint, that.textPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textAnchor, that.textAnchor)) {
      return false ;
    }
    if (this.rotationAngle != that.rotationAngle) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textHorizontalAlign, that.textHorizontalAlign)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textVerticalAlign, that.textVerticalAlign)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textPosition, that.textPosition)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textDirection, that.textDirection)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractTextAnnotation clone = (AbstractTextAnnotation)super.clone() ;

    return clone ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return this.text ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.textPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.textPaint = SerialUtil.readPaint(stream) ;
  }

}
