package com.ijchart.xychart.annotation ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.shape.TextAndPointerChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.XYPlot}ļͷע.</zh>
 * <en>A pointer annotation that can be placed on a {@link com.ijchart.xychart.plot.XYPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYPointerAnnotation extends XYTextAnnotation implements
  IXYAnnotation, Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4082080123817861415L ;

  /**
   * <zh>ȱʡļͷĽǶ(λǻ).</zh>
   * <en>The default angle of the arrow's line (in radians).</en>
   */
  public static final double DEFAULT_ARROW_ANGLE = IAngle.ANGLE_DOWN_90 ;

  /**
   * <zh>ȱʡļĳ.</zh>
   * <en>The default radius from the (x, y) point to the start of the arrow line (in Java2D units).</en>
   */
  public static final double DEFAULT_BASE_RADIUS = 30.0 ;

  /**
   * <zh>ȱʡļͷĳ.</zh>
   * <en>The default length of the arrow head (in Java2D units).</en>
   */
  public static final double DEFAULT_ARROW_LENGTH = 5 ;

  /**
   * <zh>ȱʡÿ߼ͷĿ.</zh>
   * <en>The default arrow width (in Java2D units, per side).</en>
   */
  public static final double DEFAULT_ARROW_WIDTH = 3 ;

  /**
   * <zh>ȱʡĻƼͷpaint.</zh>
   * <en>The default arrow paint.</en>
   */
  public static final Paint DEFAULT_ARROW_PAINT = Color.black ;

  /**
   * <zh>ȱʡĻƼͷstroke.</zh>
   * <en>The default arrow stroke.</en>
   */
  public static final Stroke DEFAULT_ARROW_STROKE = new BasicStroke(1) ;

  /**
   * <zh>ȱʡıͼľ.</zh>
   * <en>The default radius from the base point to the anchor point for the label.</en>
   */
  public static final double DEFAULT_LABEL_OFFSET = 3.0 ;

  /**
   * <zh>ͷĽǶ(λǻ).</zh>
   * <en>The angle of the arrow's line (in radians).</en>
   */
  private double arrowAngle ;

  /**
   * <zh>ĳ.</zh>
   * <en>The radius from the (x, y) point to the start of the arrow line (in Java2D units).</en>
   */
  private double baseRadius ;

  /**
   * <zh>ͷĳ.</zh>
   * <en>The length of the arrow head (in Java2D units).</en>
   */
  private double arrowLength ;

  /**
   * <zh>ÿ߼ͷĿ.</zh>
   * <en>The arrow width (in Java2D units, per side).</en>
   */
  private double arrowWidth ;

  /**
   * <zh>Ƽͷpaint.</zh>
   * <en>The arrow paint.</en>
   */
  private transient Paint arrowPaint ;

  /**
   * <zh>Ƽͷstroke.</zh>
   * <en>The arrow stroke.</en>
   */
  private transient Stroke arrowStroke ;

  /**
   * <zh>ıͼľ.</zh>
   * <en>The radius from the base point to the anchor point for the label.</en>
   */
  private double labelOffset ;

  /**
   * <zh>xyֵȷʾעλ.</zh>
   * <en>Creates a new annotation to be displayed at the given location.</en>
   *
   * @param text String <zh>ע (Ϊnull).</zh>
   * <en>The text (null not permitted).</en>
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param y double <zh>yֵ.</zh>
   * <en>The y value.</en>
   */
  public XYPointerAnnotation(String text, double x, double y) {
    this(text, x, y, DEFAULT_ARROW_ANGLE) ;
  }

  /**
   * <zh>xyֵȷʾעλ.</zh>
   * <en>Creates a new annotation to be displayed at the given location.</en>
   *
   * @param text String <zh>ע (Ϊnull).</zh>
   * <en>The text (null not permitted).</en>
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param y double <zh>yֵ.</zh>
   * <en>The y value.</en>
   * @param arrowAngle <zh>ͷĽǶ(λǻ).</zh>
   * <en>The angle of the arrow's line (in radians).</en>
   */
  public XYPointerAnnotation(String text, double x, double y, double arrowAngle) {
    super(text, x, y) ;

    this.arrowAngle = arrowAngle ;
    this.baseRadius = DEFAULT_BASE_RADIUS ;
    this.arrowLength = DEFAULT_ARROW_LENGTH ;
    this.arrowWidth = DEFAULT_ARROW_WIDTH ;
    this.labelOffset = DEFAULT_LABEL_OFFSET ;
    this.arrowStroke = DEFAULT_ARROW_STROKE ;
    this.arrowPaint = DEFAULT_ARROW_PAINT ;
  }

  /**
   * <zh>ͷĽǶ(λǻ).</zh>
   * <en>Return the angle of the arrow's line (in radians).</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public double getArrowAngle() {
    return this.arrowAngle ;
  }

  /**
   * <zh>üͷĽǶ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the angle of the arrow's line and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param angle double <zh>Ƕ (λǻ).</zh>
   * <en>The angle (in radians).</en>
   */
  public void setArrowAngle(double angle) {
    if (this.arrowAngle != angle) {
      this.arrowAngle = angle ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ؼĳ.</zh>
   * <en>Return the radius from the (x, y) point to the start of the arrow line
   *  (in Java2D units).</en>
   *
   * @return double <zh>ĳ.</zh>
   * <en>The radius of arrow.</en>
   */
  public double getBaseRadius() {
    return this.baseRadius ;
  }

  /**
   * <zh>üĳ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the radius from the (x, y) point to the start of the arrow line (in Java2D units) and
   *  sends a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to
   *  all registered listeners.</en>
   *
   * @param radius double <zh>ĳ.</zh>
   * <en>The radius of arrow.</en>
   */
  public void setBaseRadius(double radius) {
    if (this.baseRadius != radius) {
      this.baseRadius = radius ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ؼͷĳ.</zh>
   * <en>Return the length of the arrow head (in Java2D units).</en>
   *
   * @return double <zh>ͷĳ.</zh>
   * <en>The arrow length.</en>
   */
  public double getArrowLength() {
    return this.arrowLength ;
  }

  /**
   * <zh>üͷĳ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the length of the arrow head (in Java2D units) and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param length double <zh>ͷĳ.</zh>
   * <en>The arrow length.</en>
   */
  public void setArrowLength(double length) {
    if (this.arrowLength != length) {
      this.arrowLength = length ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ÿ߼ͷĿ.</zh>
   * <en>Return the arrow width (in Java2D units, per side).</en>
   *
   * @return double <zh>ͷĿ.</zh>
   * <en>The arrow width.</en>
   */
  public double getArrowWidth() {
    return this.arrowWidth ;
  }

  /**
   * <zh>ÿ߼ͷĿ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the arrow width (in Java2D units, per side) and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param width double <zh>ͷĿ.</zh>
   * <en>The arrow width.</en>
   */
  public void setArrowWidth(double width) {
    if (this.arrowWidth != width) {
      this.arrowWidth = width ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ıͼľ.</zh>
   * <en>Return the radius from the base point to the anchor point for the label.</en>
   *
   * @return double <zh>ıͼľ.</zh>
   * <en>The radius from the base point to the anchor point for the label.</en>
   */
  public double getLabelOffset() {
    return this.labelOffset ;
  }

  /**
   * <zh>ıͼľ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the radius from the base point to the anchor point for the label and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ıͼľ.</zh>
   * <en>The radius from the base point to the anchor point for the label.</en>
   */
  public void setLabelOffset(double offset) {
    if (this.labelOffset != offset) {
      this.labelOffset = offset ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ػƼͷpaint.</zh>
   * <en>Returns the arrow paint.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The arrow paint (never null).</en>
   */
  public Paint getArrowPaint() {
    return this.arrowPaint ;
  }

  /**
   * <zh>ûƼͷpaint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the arrow paint and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The arrow paint (null not permitted).</en>
   */
  public void setArrowPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.arrowPaint = paint ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ػƼͷstroke.</zh>
   * <en>Returns the arrow stroke.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The arrow stroke (never null).</en>
   */
  public Stroke getArrowStroke() {
    return this.arrowStroke ;
  }

  /**
   * <zh>ûƼͷstroke,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the arrow stroke and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The arrow stroke (null not permitted).</en>
   */
  public void setArrowStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.arrowStroke = stroke ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param xAxis AbstractValueAxis <zh>x (Ϊnull).</zh>
   * <en>The x-axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ (Ϊnull).</zh>
   * <en>The x-axis edge (null not permitted).</en>
   * @param yAxis AbstractValueAxis <zh>y (Ϊnull).</zh>
   * <en>The y-axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y-axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                             AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                             Rectangle2D dataArea,
                             ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    double xPos = xAxis.valueToJava2D(dataArea, xAxisEdge, getX(), false) ;
    if (xPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double yPos = yAxis.valueToJava2D(dataArea, yAxisEdge, getY(), false) ;
    if (yPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    //ȻƼͷ
    GeneralPath arrowPath = new GeneralPath() ;
    arrowPath.moveTo(0, 0) ;
    arrowPath.lineTo((float)this.arrowLength, (float) - this.arrowWidth) ;
    arrowPath.lineTo((float)this.arrowLength, (float)this.arrowWidth) ;
    arrowPath.closePath() ;

    Line2D arrowLine = new Line2D.Double(this.arrowLength, 0, this.arrowLength + this.baseRadius, 0) ;
    arrowPath.append(arrowLine, false) ;

    AffineTransform transform = new AffineTransform() ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      transform.translate(xPos, yPos) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      transform.translate(yPos, xPos) ;
    }
    transform.rotate(this.arrowAngle) ;
    Shape arrowShape = arrowPath.createTransformedShape(transform) ;

    //ַľ,ȼλ
    double totalArrowLen = this.arrowLength + this.baseRadius + this.labelOffset ;
    double circleX = totalArrowLen * Math.cos(this.arrowAngle) ;
    double circleY = totalArrowLen * Math.sin(this.arrowAngle) ;
    int quadrant = AngleUtil.getQuadrantByRadianAngle(this.arrowAngle) ; //Ƕ

    double textHeight = 0 ; //߶
    if (quadrant == 3 || quadrant == 3) {
      textHeight = -1 * TextUtil.getStringNoWrapBounds(getText(), getTextFont()).getHeight() ;
    }

    Rectangle2D textRect = null ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      textRect = new Rectangle2D.Double(xPos + circleX, yPos + circleY + textHeight, 1, 1) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      textRect = new Rectangle2D.Double(yPos + circleX, xPos + circleY + textHeight, 1, 1) ;
    }

    TextAndPointerChartShape annotation = new TextAndPointerChartShape(textRect, getText(),
                                                                       arrowShape) ;
    annotation.setDrawBorder(false) ;
    annotation.setTextPaint(getTextPaint()) ;
    annotation.setFillShape(false) ;
    annotation.setRotate(getRotationAngle()) ;
    annotation.setForceShowAllText(true) ;
    annotation.setArrowPaint(this.arrowPaint) ;
    annotation.setTooltipText(getTooltipText()) ;
    annotation.setArrowStroke(this.arrowStroke) ;

    annotation.setTextHorizontalAlign(getTextHorizontalAlign()) ;
    annotation.setTextVerticalAlign(getTextVerticalAlign()) ;
    annotation.setTextPosition(getTextPosition()) ;
    annotation.setTextDirection(getTextDirection()) ;

    annotationShapes.addAnnotationShape(annotation) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYPointerAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYPointerAnnotation that = (XYPointerAnnotation) obj ;
    if (this.arrowAngle != that.arrowAngle) {
      return false ;
    }
    if (this.baseRadius != that.baseRadius) {
      return false ;
    }
    if (this.arrowLength != that.arrowLength) {
      return false ;
    }
    if (this.arrowWidth != that.arrowWidth) {
      return false ;
    }
    if (this.labelOffset != that.labelOffset) {
      return false ;
    }
    if (!EqualsUtil.equals(this.arrowPaint, that.arrowPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.arrowStroke, that.arrowStroke)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + 100 * this.arrowAngle ;
    hashcode = 37 * hashcode + 100 * this.baseRadius ;
    hashcode = 37 * hashcode + 100 * this.arrowLength ;
    hashcode = 37 * hashcode + 100 * this.arrowWidth ;
    hashcode = 37 * hashcode + 100 * this.labelOffset ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.arrowStroke) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.arrowPaint) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[arrowAngle=" + this.arrowAngle
      + ",baseRadius=" + this.baseRadius
      + ",arrowLength=" + this.arrowLength
      + ",arrowWidth=" + this.arrowWidth
      + ",labelOffset=" + this.labelOffset
      + "]" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.arrowPaint) ;
    SerialUtil.writeStroke(stream, this.arrowStroke) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.arrowPaint = SerialUtil.readPaint(stream) ;
    this.arrowStroke = SerialUtil.readStroke(stream) ;
  }
}
