package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XY plotʹõֵ,Ҫѡ͵ĸ,ÿһ
 * {@link com.ijchart.xychart.data.IXYChartData}.</zh>
 * <en>The abstract parent class of number value axis used for XY plot,every data is
 *  {@link com.ijchart.xychart.data.IXYChartData}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AbstractNumberValueAxisWithType extends NumberValueAxis implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1868873737599640990L ;

  /**
   * <zh>value.</zh>
   * <en>The type of value axis.</en>
   */
  private XYValueAxisType valueAxisType ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   */
  public AbstractNumberValueAxisWithType(IMatrixDataset dataset, XYValueAxisType valueAxisType) {
    this(null, dataset, valueAxisType, false) ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public AbstractNumberValueAxisWithType(IMatrixDataset dataset, XYValueAxisType valueAxisType,
                                         boolean isDateAxis) {
    this(null, dataset, valueAxisType, isDateAxis) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   */
  public AbstractNumberValueAxisWithType(String label, IMatrixDataset dataset,
                                         XYValueAxisType valueAxisType) {
    this(label, dataset, valueAxisType, false) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public AbstractNumberValueAxisWithType(String label, IMatrixDataset dataset,
                                         XYValueAxisType valueAxisType,
                                         boolean isDateAxis) {
    super(label, dataset) ;
    if (valueAxisType == null) {
      throw new IllegalArgumentException("Null 'valueAxisType' argument.") ;
    }

    this.valueAxisType = valueAxisType ;
    if (isDateAxis) {
      this.tickUnit = new DateTickUnit(null) ;
    }
  }

  /**
   * <zh>value.</zh>
   * <en>Returns the type of value axis.</en>
   *
   * @return XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (never null).</en>
   */
  public XYValueAxisType getValueAxisType() {
    return this.valueAxisType ;
  }

  /**
   * <zh>value,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the type of value axis,and notifies registered listeners
   * that the axis has changed.</en>
   *
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   */
  public void setValueAxisType(XYValueAxisType valueAxisType) {
    if (valueAxisType == null) {
      throw new IllegalArgumentException("Null 'valueAxisType' argument.") ;
    }

    if (!EqualsUtil.equals(this.valueAxisType, valueAxisType)) {
      this.valueAxisType = valueAxisType ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof AbstractNumberValueAxisWithType)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractNumberValueAxisWithType that = (AbstractNumberValueAxisWithType) obj ;
    if (!EqualsUtil.equals(this.valueAxisType, that.valueAxisType)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.valueAxisType) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractNumberValueAxisWithType" ;
  }

}
