package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;
import java.util.Date ;

import com.ijchart.xychart.data.OHLCChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.tickunit.DateTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>OHLCֵ.</zh>
 * <en> The class for axes that display dates of OHLCChartData.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class OHLCDateAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -796643112864369826L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset SingleRowDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public OHLCDateAxis(SingleRowDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset SingleRowDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public OHLCDateAxis(String label, SingleRowDataset dataset) {
    super(label, dataset) ;

    this.tickUnit = new DateTickUnit(null) ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;

    if (dataset != null) {
      int rows = dataset.getRowCount() ;

      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof OHLCChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'OHLCChartData'.") ;
            } else {
              OHLCChartData ohlcData = (OHLCChartData) data ;
              Date date = ohlcData.getDate() ;
              if (date != null) {
                if (Double.isNaN(minValue)) {
                  minValue = date.getTime() ;
                } else {
                  minValue = Math.min(minValue, date.getTime()) ;
                }

                if (Double.isNaN(maxValue)) {
                  maxValue = date.getTime() ;
                } else {
                  maxValue = Math.max(maxValue, date.getTime()) ;
                }

              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof OHLCDateAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "OHLCDateAxis" ;
  }

}
