package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢ϵͼεļ.</zh>
 * <en>Collection to be used to store the shapes in axis.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AxisShapeCollection extends AbstractShapeCollection implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 2021786712035447236L ;

  /**
   * <zh>λ.</zh>
   * <en>The axis edge.</en>
   */
  private RectangleEdge edge ;

  /**
   * <zh>ǩͼμ.</zh>
   * <en>The axis label shape.</en>
   */
  private List axisLabelShape ;

  /**
   * <zh>ͼμ.</zh>
   * <en>The axis line shape.</en>
   */
  private List axisLineShape ;

  /**
   * <zh>3D͵򼯺.</zh>
   * <en>The axis line 3D area.</en>
   */
  private List axisLine3DShape ;

  /**
   * <zh>̶ȱǩͼμ.</zh>
   * <en>The list of axis tick label shapes.</en>
   */
  private List tickLabelShape ;

  /**
   * <zh>̶ͼμ.</zh>
   * <en>The list of axis tick mark shapes.</en>
   */
  private List tickMarkShape ;

  /**
   * <zh>޲.</zh>
   * <en>No argument constructor.</en>
   */
  public AxisShapeCollection() {
    this.edge = null ;
    this.axisLabelShape = new ArrayList() ;
    this.axisLineShape = new ArrayList() ;
    this.axisLine3DShape = new ArrayList() ;
    this.tickLabelShape = new ArrayList() ;
    this.tickMarkShape = new ArrayList() ;
  }

  /**
   * <zh>λ.</zh>
   * <en>Returns the axis edge.</en>
   *
   * @return RectangleEdge <zh>λ.</zh>
   * <en>The axis edge.</en>
   */
  public RectangleEdge getEdge() {
    return this.edge ;
  }

  /**
   * <zh>λ.</zh>
   * <en>Sets the axis edge.</en>
   *
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis edge.</en>
   */
  public void setEdge(RectangleEdge edge) {
    this.edge = edge ;
  }

  /**
   * <zh>ǩͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of axis label shapes and is unmodifiable.</en>
   *
   * @return List <zh>ǩͼμ(Ϊnull).</zh>
   * <en>The list of axis label shape (never null).</en>
   */
  public List getAxisLabelShapes() {
    return Collections.unmodifiableList(this.axisLabelShape) ;
  }

  /**
   * <zh>ɾеǩͼ.</zh>
   * <en>Remove all axis label shapes.</en>
   */
  public void clearAxisLabelShapes() {
    removeShapesListener(this.axisLabelShape, this) ;
    this.axisLabelShape.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һǩͼ.</zh>
   * <en>Add an axis label shape.</en>
   *
   * @param axisLabelShape IChartShape <zh>ǩͼ(Ϊnull).</zh>
   * <en>The axis label shape (null permitted).</en>
   */
  public void addAxisLabelShape(IChartShape axisLabelShape) {
    if (axisLabelShape != null) {
      axisLabelShape.setId(shapeIdSequence++) ;
      axisLabelShape.setShapeDrawType(ChartShapeDrawType.AXIS_LABEL_SHAPE) ;
      axisLabelShape.addChangeListener(this) ;
    }

    this.axisLabelShape.add(axisLabelShape) ;
    fireChartShapeChanged(axisLabelShape) ;
  }

  /**
   * <zh>һͼ.</zh>
   * <en>Add an axis line shape.</en>
   *
   * @param axisLineShape IChartShape <zh>ͼ(Ϊnull).</zh>
   * <en>The axis line shape (null permitted).</en>
   */
  public void addAxisLineShape(IChartShape axisLineShape) {
    if (axisLineShape != null) {
      axisLineShape.setId(shapeIdSequence++) ;
      axisLineShape.setShapeDrawType(ChartShapeDrawType.AXIS_LINE_SHAPE) ;
      axisLineShape.addChangeListener(this) ;
    }

    this.axisLineShape.add(axisLineShape) ;
    fireChartShapeChanged(axisLineShape) ;
  }

  /**
   * <zh>ͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of axis line shape and is unmodifiable.</en>
   *
   * @return List <zh>ͼμ(Ϊnull).</zh>
   * <en>The list of axis line shape (never null).</en>
   */
  public List getAxisLineShapes() {
    return Collections.unmodifiableList(this.axisLineShape) ;
  }

  /**
   * <zh>ɾеͼ.</zh>
   * <en>Remove all axis line shapes.</en>
   */
  public void clearAxisLineShapes() {
    removeShapesListener(this.axisLineShape, this) ;
    this.axisLineShape.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ3D͵.</zh>
   * <en>Add an axis line 3D area shape.</en>
   *
   * @param line3DShape IChartShape <zh>3D͵ (Ϊnull).</zh>
   * <en>The axis line 3D area (null permitted).</en>
   */
  public void addAxisLine3DShape(IChartShape line3DShape) {
    if (line3DShape != null) {
      line3DShape.setId(shapeIdSequence++) ;
      line3DShape.setShapeDrawType(ChartShapeDrawType.AXIS_LINE_3D_SHAPE) ;
      line3DShape.addChangeListener(this) ;
    }

    this.axisLine3DShape.add(line3DShape) ;
    fireChartShapeChanged(line3DShape) ;
  }

  /**
   * <zh>3D͵ͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of axis line 3D area and is unmodifiable.</en>
   *
   * @return List <zh>3D͵ͼμ (Ϊnull).</zh>
   * <en>The list of axis line 3D area (never null).</en>
   */
  public List getAxisLine3DShapes() {
    return Collections.unmodifiableList(this.axisLine3DShape) ;
  }

  /**
   * <zh>ɾе3D͵ͼ.</zh>
   * <en>Remove all axis line 3D area shapes.</en>
   */
  public void clearAxisLine3DShapes() {
    removeShapesListener(this.axisLine3DShape, this) ;
    this.axisLine3DShape.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ̶ȱǩͼ.</zh>
   * <en>Add an axis tick label shape.</en>
   *
   * @param labelShape IChartShape <zh>̶ȱǩͼ (Ϊnull).</zh>
   * <en>An axis tick label shape (null permitted).</en>
   */
  public void addTickLabelShape(IChartShape labelShape) {
    if (labelShape != null) {
      labelShape.setId(shapeIdSequence++) ;
      labelShape.setShapeDrawType(ChartShapeDrawType.AXIS_TICK_LABEL_SHAPE) ;
      labelShape.addChangeListener(this) ;
    }

    this.tickLabelShape.add(labelShape) ;
    fireChartShapeChanged(labelShape) ;
  }

  /**
   * <zh>̶ȱǩͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of axis tick label shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>̶ȱǩͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of axis tick label shapes and is unmodifiable.</en>
   */
  public List getTickLabelShapes() {
    return Collections.unmodifiableList(this.tickLabelShape) ;
  }

  /**
   * <zh>һ̶ֵͼ.</zh>
   * <en>Add an axis tick mark shape.</en>
   *
   * @param markShape IChartShape <zh>̶ֵͼ (Ϊnull).</zh>
   * <en>An axis tick mark shape (null permitted).</en>
   */
  public void addTickMarkShape(IChartShape markShape) {
    if (markShape != null) {
      markShape.setId(shapeIdSequence++) ;
      markShape.setShapeDrawType(ChartShapeDrawType.AXIS_TICK_MARK_SHAPE) ;
      markShape.addChangeListener(this) ;
    }

    this.tickMarkShape.add(markShape) ;
    fireChartShapeChanged(markShape) ;
  }

  /**
   * <zh>̶ֵͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of axis tick mark shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>̶ֵͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of axis tick mark shapes and is unmodifiable.</en>
   */
  public List getTickMarkShapes() {
    return Collections.unmodifiableList(this.tickMarkShape) ;
  }

  /**
   * <zh>ɾе̶ֵͼ.</zh>
   * <en>Remove all tick mark shapes.</en>
   */
  public void clearTickMarkShapes() {
    removeShapesListener(this.tickMarkShape, this) ;
    this.tickMarkShape.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    draw(g2, this.axisLineShape) ;
    draw(g2, this.axisLine3DShape) ;
    draw(g2, this.axisLabelShape) ;
    draw(g2, this.tickMarkShape) ;
    draw(g2, this.tickLabelShape) ;
  }

  /**
   * <zh>ƶͼ(axisLine3DShape)ԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shapes(not include axisLine3DShape) in this chart to the
   * point (x,&nbsp;y) in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    translate(x, y, this.axisLineShape) ;
    translate(x, y, this.axisLine3DShape) ;
    translate(x, y, this.axisLabelShape) ;
    translate(x, y, this.tickMarkShape) ;
    translate(x, y, this.tickLabelShape) ;
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    clearShapesListener(this.axisLineShape) ;
    clearShapesListener(this.axisLine3DShape) ;
    clearShapesListener(this.axisLabelShape) ;
    clearShapesListener(this.tickMarkShape) ;
    clearShapesListener(this.tickLabelShape) ;
  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.axisLineShape) ;
    clearSelectedShape(this.axisLine3DShape) ;
    clearSelectedShape(this.axisLabelShape) ;
    clearSelectedShape(this.tickMarkShape) ;
    clearSelectedShape(this.tickLabelShape) ;
  }

  /**
   * <zh>һͼε.</zh>
   * <en>Add all shapes of a AxisShapeCollection to this object.</en>
   *
   * @param axisShapes AxisShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>The AxisShapeCollection (null permitted).</en>
   */
  public void copyShapes(AxisShapeCollection axisShapes) {
    if (axisShapes != null) {
      this.axisLabelShape.addAll(axisShapes.getAxisLabelShapes()) ;
      this.axisLineShape.addAll(axisShapes.getAxisLineShapes()) ;
      this.axisLine3DShape.addAll(axisShapes.getAxisLine3DShapes()) ;
      this.tickMarkShape.addAll(axisShapes.getTickMarkShapes()) ;
      this.tickLabelShape.addAll(axisShapes.getTickLabelShapes()) ;
      fireChartShapeChanged(null) ;
    }
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    IChartShape shape = null ;
    if (flag || drawType.equals(ChartShapeDrawType.AXIS_TICK_LABEL_SHAPE)) {
      shape = findShape(this.tickLabelShape, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.AXIS_TICK_MARK_SHAPE)) {
      shape = findShape(this.tickMarkShape, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.AXIS_LINE_3D_SHAPE)) {
      shape = findShape(this.axisLine3DShape, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.AXIS_LINE_SHAPE)) {
      shape = findShape(this.axisLineShape, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.AXIS_LABEL_SHAPE)) {
      shape = findShape(this.axisLabelShape, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    if (drawTypes == null) {
      throw new IllegalArgumentException("Null 'drawTypes' argument.") ;
    }

    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.AXIS_TICK_LABEL_SHAPE)) {
      addHotspotArea(result, this.tickLabelShape, fragment,
                     ChartShapeDrawType.AXIS_TICK_LABEL_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.AXIS_TICK_MARK_SHAPE)) {
      addHotspotArea(result, this.tickMarkShape, fragment, ChartShapeDrawType.AXIS_TICK_MARK_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.AXIS_LINE_3D_SHAPE)) {
      addHotspotArea(result, this.axisLine3DShape, fragment, ChartShapeDrawType.AXIS_LINE_3D_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.AXIS_LINE_SHAPE)) {
      addHotspotArea(result, this.axisLineShape, fragment, ChartShapeDrawType.AXIS_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.AXIS_LABEL_SHAPE)) {
      addHotspotArea(result, this.axisLabelShape, fragment, ChartShapeDrawType.AXIS_LABEL_SHAPE) ;
    }

    return result ;
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.axisLabelShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.edge) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLineShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLine3DShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickMarkShape) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AxisShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AxisShapeCollection that = (AxisShapeCollection) obj ;
    if (!EqualsUtil.equals(this.edge, that.edge)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelShape, that.axisLabelShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLineShape, that.axisLineShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLine3DShape, that.axisLine3DShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelShape, that.tickLabelShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickMarkShape, that.tickMarkShape)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[edge=" + this.edge
      + ",labelShape size=" + this.axisLabelShape.size()
      + ",axisLineShape size=" + this.axisLineShape.size()
      + ",axisLine3DShape size=" + this.axisLine3DShape.size()
      + ",tickLabelShape size=" + this.tickLabelShape.size()
      + ",tickMarkShape size=" + this.tickMarkShape.size()
      + "]" ;
  }

}
