package com.ijchart.xychart.constant ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>plotͳ.</zh>
 * <en>Define the type of a plot object.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class PlotType implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -7308786811045175737L ;

  /**
   * <zh>CategoryPlot.</zh>
   * <en>The type of CategoryPlot.</en>
   */
  public static final PlotType PLOT_TYPE_CATEGORY = new PlotType("PlotType.CategoryPlot") ;

  /**
   * <zh>CategoryPlot3D.</zh>
   * <en>The type of CategoryPlot3D.</en>
   */
  public static final PlotType PLOT_TYPE_CATEGORY3D = new PlotType("PlotType.CategoryPlot3D") ;

  /**
   * <zh>XYPlot.</zh>
   * <en>The type of XYPlot.</en>
   */
  public static final PlotType PLOT_TYPE_XYPLOT = new PlotType("PlotType.XYPlot") ;

  /**
   * <zh>ûplot.</zh>
   * <en>The type of NO axis plot.</en>
   */
  public static final PlotType PLOT_TYPE_NO_AXIS = new PlotType("PlotType.NO_AXIS") ;

  /**
   * <zh>CombineCategoryAxisCategoryPlot.</zh>
   * <en>The type of CombineCategoryAxisCategoryPlot.</en>
   */
  public static final PlotType PLOT_TYPE_COMBINE_CATEGORY_AXIS_CATEGORY_PLOT = new PlotType(
    "PlotType.CombineCategoryAxisCategoryPlot") ;

  /**
   * <zh>CombineValueAxisCategoryPlot.</zh>
   * <en>The type of CombineValueAxisCategoryPlot.</en>
   */
  public static final PlotType PLOT_TYPE_COMBINE_VALUE_AXIS_CATEGORY_PLOT = new PlotType(
    "PlotType.CombineValueAxisCategoryPlot") ;

  /**
   * <zh>CombineDomainAxisXYPlot.</zh>
   * <en>The type of CombineDomainAxisXYPlot.</en>
   */
  public static final PlotType PLOT_TYPE_COMBINE_DOMAIN_AXIS_XY_PLOT = new PlotType(
    "PlotType.CombineDomainAxisXYPlot") ;

  /**
   * <zh>CombineValueAxisXYPlot.</zh>
   * <en>The type of CombineValueAxisXYPlot.</en>
   */
  public static final PlotType PLOT_TYPE_COMBINE_VALUE_AXIS_XY_PLOT = new PlotType(
    "PlotType.CombineValueAxisXYPlot") ;

  /**
   * <zh>.</zh>
   * <en>The name.</en>
   */
  private String name ;

  /**
   * <zh>˽йֹʵ.</zh>
   * <en>Private constructor to prevent instance.</en>
   *
   * @param name <zh>.</zh>
   * <en>The name.</en>
   */
  private PlotType(String name) {
    this.name = name ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof PlotType)) {
      return false ;
    }

    PlotType tmp = (PlotType) obj ;

    return this.name.equals(tmp.name) ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[name=" + name + "]" ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object. </en>
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    return (this.name == null) ? 0 : this.name.hashCode() ;
  }

}
