package com.ijchart.xychart.data ;

import java.io.Serializable ;
import java.util.Date ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>(,,ǿ)ֵͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>The (date,windDirection,windForce) ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class WindChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 589040606021635483L ;

  /**
   * <zh>().</zh>
   * <en>The date (in millseconds).</en>
   */
  private Number date ;

  /**
   * <zh>ֵ(ָ0-12ӦıϵĿ̶ֵ).</zh>
   * <en>The wind direction (should be in the range 0 to 12,
   * corresponding to the positions on an upside-down clock face).</en>
   */
  private Number windDirection ;

  /**
   * <zh>ǿ.</zh>
   * <en>The wind force.</en>
   */
  private Number windForce ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param date long <zh>().</zh>
   * <en>The date (in millseconds).</en>
   * @param windDirection double <zh>ֵ(ָ0-12ӦıϵĿ̶ֵ).</zh>
   * <en>The wind direction (should be in the range 0 to 12,
   * corresponding to the positions on an upside-down clock face).</en>
   * @param windForce double <zh>ǿ,ڵ0.</zh>
   * <en>The wind force and must be >=0.</en>
   */
  public WindChartData(long date, double windDirection, double windForce) {
    this(new Double(date), new Double(windDirection), new Double(windForce)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param date Number <zh>() (Ϊnull).</zh>
   * <en>The date (in millseconds) (null not permitted).</en>
   * @param windDirection Number <zh>ֵ(ָ0-12ӦıϵĿ̶ֵ).</zh>
   * <en>The wind direction (should be in the range 0 to 12,
   * corresponding to the positions on an upside-down clock face).</en>
   * @param windForce Number <zh>ǿ,ڵ0.</zh>
   * <en>The wind force and must be >=0.</en>
   */
  public WindChartData(Number date, Number windDirection, Number windForce) {
    if (date == null) {
      throw new IllegalArgumentException("Null 'date' argument") ;
    }
    if (windDirection == null) {
      throw new IllegalArgumentException("Null 'windDirection' argument.") ;
    }
    if (windForce == null) {
      throw new IllegalArgumentException("Null 'windForce' argument.") ;
    }
    if (windForce.doubleValue() < 0) {
      throw new IllegalArgumentException("Argument 'windForce' must be >=0.") ;
    }

    this.date = date ;
    this.windDirection = windDirection ;
    this.windForce = windForce ;
  }

  /**
   * <zh>()Ķ.</zh>
   * <en>Returns the date object.</en>
   *
   * @return double <zh>() (Ϊnull).</zh>
   * <en>The date (in millseconds) (never null).</en>
   */
  public Number getDate() {
    return this.date ;
  }

  /**
   * <zh>().</zh>
   * <en>Returns the date value.</en>
   *
   * @return double <zh>() (Ϊnull).</zh>
   * <en>The date (in millseconds) (never null).</en>
   */
  public double getDateValue() {
    return this.date.doubleValue() ;
  }

  /**
   * <zh>طǿֵĶ.</zh>
   * <en>Returns the wind force data.</en>
   *
   * @return double <zh>ǿ.</zh>
   * <en>The wind force.</en>
   */
  public Number getWindForceData() {
    return this.windForce ;
  }

  /**
   * <zh>طǿֵ.</zh>
   * <en>Returns the wind force.</en>
   *
   * @return double <zh>ǿ.</zh>
   * <en>The wind force.</en>
   */
  public double getWindForce() {
    return this.windForce.doubleValue() ;
  }

  /**
   * <zh>طֵ(λǽǶ).</zh>
   * <en>Returns the wind direction data (in degree).</en>
   *
   * @return double <zh>ֵ(λǽǶ).</zh>
   * <en>The wind direction (in degree).</en>
   */
  public Number getWindDirectionData() {
    return this.windDirection ;
  }

  /**
   * <zh>طֵ(λǽǶ).</zh>
   * <en>Returns the wind direction (in degree).</en>
   *
   * @return double <zh>ֵ(λǽǶ).</zh>
   * <en>The wind direction (in degree).</en>
   */
  public double getWindDirection() {
    return this.windDirection.doubleValue() ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof XYNumberChartData)) {
        throw new ClassCastException("Must be 'XYNumberChartData'.") ;
      }

      WindChartData other = (WindChartData) o ;
      double diff = this.date.doubleValue() - other.getDateValue() ;
      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (never null).</en>
   */
  public String getLabelText() {
    return "(" + formatDate(new Date((long) getDateValue())) + "," + formatNumber(getWindDirection())
      + "," + formatNumber(getWindForce()) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatDateForFunction(new Date((long) getDateValue()))
      + "," + formatNumberForFunction(getWindDirection())
      + "," + formatNumberForFunction(getWindForce()) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "date=" + formatDate(new Date((long) getDateValue())) + "&windDirection="
      + formatNumber(getWindDirection()) + "&windForce=" + formatNumber(getWindForce()) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof WindChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    WindChartData that = (WindChartData) obj ;
    if (EqualsUtil.equals(this.date, that.date)) {
      return false ;
    }
    if (EqualsUtil.equals(this.windDirection, that.windDirection)) {
      return false ;
    }
    if (EqualsUtil.equals(this.windForce, that.windForce)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.date) ;
    result = 37 * result + HashcodeUtil.hashCode(this.windDirection) ;
    result = 37 * result + HashcodeUtil.hashCode(this.windForce) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[date=" + getDateValue()
      + ",windDirection=" + getWindDirection()
      + ",windForce=" + getWindForce()
      + "]" ;
  }

}
