package com.ijchart.xychart.dataset.data ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.KeyValueComparatorType ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ֵֵӦϵ.ֵֵɿ¡ҿл.</zh>
 * <en>Class for represent (key,value) pair.The key and value object must implement Cloneable and
 * Serializable interface.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class KeyValuePair implements Comparable, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4355922015267764404L ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>The default comparator type.</en>
   */
  public static final KeyValueComparatorType DEFAULT_COMPARATOR_TYPE = KeyValueComparatorType.
                                                                       BY_KEY ;

  /**
   * <zh>ֵ.</zh>
   * <en>The key value.</en>
   */
  private Object key ;

  /**
   * <zh>ֵӦֵ.</zh>
   * <en>The value.</en>
   */
  private Object value ;

  /**
   * <zh>.</zh>
   * <en>The comparator type.</en>
   */
  private KeyValueComparatorType comparatorType ;

  /**
   * <zh>ɼֵֵ.</zh>
   * <en>Construct a instance by (key,value) pair.</en>
   *
   * @param key Object <zh>ֵ(Ϊnull)</zh>
   * <en>The key value (null not permitted).</en>
   * @param value Object <zh>ֵӦֵ (Ϊnull).</zh>
   * <en>The value ((null permitted)).</en>
   */
  public KeyValuePair(Object key, Object value) {
    this(key, value, DEFAULT_COMPARATOR_TYPE) ;
  }

  /**
   * <zh>ɼֵֵ.</zh>
   * <en>Construct a instance by (key,value) pair.</en>
   *
   * @param key Object <zh>ֵ(Ϊnull)</zh>
   * <en>The key value (null not permitted).</en>
   * @param value Object <zh>ֵӦֵ (Ϊnull).</zh>
   * <en>The value ((null permitted)).</en>
   * @param comparatorType KeyValueComparatorType <zh>.</zh>
   * <en>The comparator type.</en>
   */
  public KeyValuePair(Object key, Object value, KeyValueComparatorType comparatorType) {
    if (key == null) {
      throw new IllegalArgumentException("Null argument 'key'.") ;
    }

    this.key = key ;
    this.value = value ;
    this.comparatorType = comparatorType ;
  }

  /**
   * <zh>õֵ.</zh>
   * <en>Get the key object.</en>
   * @return Object <zh>ֵ (Ϊnull).</zh>
   * <en>The key value (maybe null).</en>
   */
  public Object getKey() {
    return this.key ;
  }

  /**
   * <zh>õֵӦֵ.</zh>
   * <en>Get the value.</en>
   * @return Object <zh>ֵӦֵ(Ϊnull).</zh>
   * <en>The value (maybe null).</en>
   */
  public Object getValue() {
    return this.value ;
  }

  /**
   * <zh>޸ļֵӦֵ.</zh>
   * <en>Sets the value associate with the key.</en>
   * @param value Object <zh>ֵӦֵ (Ϊnull).</zh>
   * <en>The value associate with the key (null permitted).</en>
   */
  public void setValue(Object value) {
    this.value = value ;
  }

  /**
   * <zh>.</zh>
   * <en>Returns the comparator type.</en>
   *
   * @return KeyValueComparatorType <zh>.</zh>
   * <en>The comparator type.</en>
   */
  public KeyValueComparatorType getComparatorType() {
    return this.comparatorType ;
  }

  /**
   * <zh>.</zh>
   * <en>Sets the comparator type.</en>
   *
   * @param type KeyValueComparatorType <zh>.</zh>
   * <en>The comparator type.</en>
   */
  public void setComparatorType(KeyValueComparatorType type) {
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument") ;
    }

    this.comparatorType = type ;
  }

  /**
   * <zh>Ƚ.</zh>
   * <en>Compare two objects.</en>
   *
   * @param other Object
   * @return int
   */
  public int compareTo(Object other) {
    if (other == null) {
      return 1 ;
    }

    KeyValuePair otherPair = (KeyValuePair) other ;
    if (this.comparatorType.equals(KeyValueComparatorType.BY_KEY)) {
      return ((Comparable)this.key).compareTo((Comparable) otherPair.key) ;
    } else if (this.comparatorType.equals(KeyValueComparatorType.BY_VALUE)) {
      if (this.value == null) {
        return -1 ;
      } else {
        return ((Comparable)this.value).compareTo((Comparable) otherPair.value) ;
      }
    } else {
      throw new IllegalArgumentException("Unknown comparator type.") ;
    }
  }

  /**
   * <zh>Ǹequals.</zh>
   * <en>Tests this key-value pair for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ.</zh>
   * <en>the object (null permitted).</en>
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof KeyValuePair)) {
      return false ;
    }

    KeyValuePair that = (KeyValuePair) obj ;

    if ((this.key != null) ? !this.key.equals(that.key) : that.key != null) {
      return false ;
    }

    if ((this.value != null) ? !this.value.equals(that.value) : that.value != null) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hascodeֵ.</zh>
   * <en>Returns a hash code.</en>
   * @return A hash code.
   */
  public int hashCode() {
    int result = ((this.key != null) ? this.key.hashCode() : 0) ;
    result = 29 * result + ((this.value != null) ? this.value.hashCode() : 0) ;

    return result ;
  }

  /**
   * <zh>ʵclone,˴ʵּٶkeyvalueǲɸıĶ.</zh>
   * <en>Returns a clone.  It is assumed that both the key and value are
   * immutable objects, so only the references are cloned, not the objects
   * themselves.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException Not thrown by this class, but
   *         subclasses (if any) might.
   */
  public Object clone() throws CloneNotSupportedException {
    KeyValuePair clone = (KeyValuePair)super.clone() ;

    return clone ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing this instance, primarily useful for debugging.</en>
   *
   * @return A string.
   */
  public String toString() {
    return "[key=" + this.key + ",value=" + this.value + "]" ;
  }

}
