package com.ijchart.xychart.event ;

import java.io.Serializable ;
import java.util.EventObject ;

import com.ijchart.xychart.constant.ChartChangeEventType ;
import com.ijchart.xychart.plot.AbstractPlot ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>plot仯¼.</zh>
 * <en>A change event that encapsulates information about a change to an plot.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PlotChangeEvent extends EventObject implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7722880742840654073L ;

  /**
   * <zh>¼plot.</zh>
   * <en>The plot that generated the change event.</en>
   */
  private AbstractPlot plot ;

  /**
   * <zh>¼.</zh>
   * <en>The event type.</en>
   */
  private ChartChangeEventType type ;

  /**
   * <zh>һµĶ,¼Ϊ
   * {@link com.ijchart.xychart.constant.ChartChangeEventType#GENERAL}.</zh>
   * <en>Creates a new instance and the event type is
   * {@link com.ijchart.xychart.constant.ChartChangeEventType#GENERAL}.</en>
   *
   * @param plot AbstractPlot <zh>¼plot.</zh>
   * <en>The plot that generated the event.</en>
   */
  public PlotChangeEvent(AbstractPlot plot) {
    this(plot, ChartChangeEventType.GENERAL) ;
  }

  /**
   * <zh>һµĶ.</zh>
   * <en>Creates a new instance.</en>
   *
   * @param plot <zh>¼plot.</zh>
   * <en>The plot that generated the event.</en>
   */
  public PlotChangeEvent(AbstractPlot plot, ChartChangeEventType type) {
    super(plot) ;
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument.") ;
    }

    this.plot = plot ;
    this.type = type ;
  }

  /**
   * <zh>ز¼plot.</zh>
   * <en>Returns the plot that generated the event.</en>
   *
   * @return <zh>¼plot</zh>
   * <en>The plot that generated the event.</en>
   */
  public AbstractPlot getPlot() {
    return this.plot ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Returns the event type.</en>
   *
   * @return <zh>¼(Ϊnull).</zh>
   * <en>The event type(never null).</en>
   */
  public ChartChangeEventType getType() {
    return this.type ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Sets the event type.</en>
   *
   * @param type <zh>¼ (Ϊnull).</zh>
   * <en>The event type (null not permitted).</en>
   */
  public void setType(ChartChangeEventType type) {
    this.type = type ;
  }
}
