package com.ijchart.xychart.render ;

import java.awt.Font ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.label.ILabelGenerator ;
import com.ijchart.xychart.listener.IRenderChangeListener ;
import com.ijchart.xychart.tooltip.IToolTipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڻͼеrenderеͼεĽӿ,ȷݼһĳʱȡitem,column items
 * Դ֮,base itemٴ֮.</zh>
 * <en>Interface for drawing shapes of the chart render area,to determine the attributes of a
 * dataset item first is item attribute, column items attribute is the second,
 * base item's attribute is third.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public interface IRender {
  /**
   * <zh>ĳĳеǷʾ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified item should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  boolean isItemVisible(int row, int column) ;

  /**
   * <zh>ĳĳеǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.
   * ͨ˷ݼĳĳеǷʾ.</zh>
   * <en>Sets a boolean that indicates whether or not the specified item should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setItemVisible(int row, int column, boolean visible) ;

  /**
   * <zh>ݼһеǷȫɼ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified column should be drawn.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isColumnItemVisible(int column) ;

  /**
   * <zh>ݼһǷȫʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.˷ڿݼ
   * еһǷʾ.</zh>
   * <en>Sets the flag that controls whether a column is visible and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.This is typically used to hide an entire column data.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The boolean flag.</en>
   */
  void setColumnItemVisible(int column, boolean visible) ;

  /**
   * <zh>ݼһеǷȫɼ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified row should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isRowItemVisible(int row) ;

  /**
   * <zh>ݼһǷȫʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.˷ڿݼ
   * еһǷʾ.</zh>
   * <en>Sets the flag that controls whether a row is visible and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.This is typically used to hide an entire row data.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The boolean flag.</en>
   */
  void setRowItemVisible(int row, boolean visible) ;

  /**
   * <zh>Ƿͼڲ.</zh>
   * <en>Returns whether or not fill shape.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  boolean isItemFill(int row, int column) ;

  /**
   * <zh>Ƿͼڲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not fill shape and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  void setItemFill(int row, int column, boolean flag) ;

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Returns whether or not to fill all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ֵ.</zh>
   * <en>The Boolean object.</en>
   */
  Boolean isColumnItemFill(int column) ;

  /**
   * <zh>ݼеһǷͼڲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to fill all items of a dataset column and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  void setColumnItemFill(int column, boolean flag) ;

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Returns whether or not to fill all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ֵ.</zh>
   * <en>The Boolean object.</en>
   */
  Boolean isRowItemFill(int row) ;

  /**
   * <zh>ݼеһǷͼڲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to fill all items of a dataset row and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  void setRowItemFill(int row, boolean flag) ;

  /**
   * <zh>Ƿͼڲ.</zh>
   * <en>Returns whether or not fill all items of a dataset.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  boolean isBaseItemFill() ;

  /**
   * <zh>Ƿͼڲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not fill all items of a dataset and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  void setBaseItemFill(boolean flag) ;

  /**
   * <zh>ͼڲpaint.</zh>
   * <en>Returns the paint used to fill data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  Paint getItemFillPaint(int row, int column) ;

  /**
   * <zh>ͼڲpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill data items as they are drawn and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  void setItemFillPaint(int row, int column, Paint paint) ;

  /**
   * <zh>ݼеһõpaint.</zh>
   * <en>Returns the paint used to fill all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  Paint getColumnItemFillPaint(int column) ;

  /**
   * <zh>ݼеһõpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill all items of a dataset column and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  void setColumnItemFillPaint(int column, Paint paint) ;

  /**
   * <zh>ݼеһõpaint.</zh>
   * <en>Returns the paint used to fill all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  Paint getRowItemFillPaint(int row) ;

  /**
   * <zh>ݼеһõpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill all items of a dataset row and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  void setRowItemFillPaint(int row, Paint paint) ;

  /**
   * <zh>õpaint.</zh>
   * <en>Returns the base fill paint.</en>
   *
   * @return Paint <zh>õpaint (Ϊnull).</zh>
   * <en>The base fill paint (null permitted).</en>
   */
  Paint getBaseItemFillPaint() ;

  /**
   * <zh>õpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base paint to fill all items of a dataset and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The base fill paint (null permitted).</en>
   */
  void setBaseItemFillPaint(Paint paint) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,򷵻ͼڲķʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit an item.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (maybe null).</en>
   */
  GradientPaintTransformType getItemFillPaintTransformer(int row, int column) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ͼڲķʽ.
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit an item,and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType
   * <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (null permitted).</en>
   */
  void setItemFillPaintTransformer(int row, int column, GradientPaintTransformType transformer) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (possibly null).</en>
   */
  GradientPaintTransformType getColumnItemFillPaintTransformer(int column) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ.
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset column,and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType <zh>paint (Ϊnull).</zh>
   * <en>The GradientPaintTransformType (null permitted).</en>
   */
  void setColumnItemFillPaintTransformer(int column, GradientPaintTransformType transformer) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (possibly null).</en>
   */
  GradientPaintTransformType getRowItemFillPaintTransformer(int row) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ.
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset row,and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType <zh>paint (Ϊnull).</zh>
   * <en>The GradientPaintTransformType (null permitted).</en>
   */
  void setRowItemFillPaintTransformer(int row, GradientPaintTransformType transformer) ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼݹõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset.</en>
   *
   * @return GradientPaintTransformType <zh>õpaint (Ϊnull).</zh>
   * <en>The base GradientPaintTransformType object (possibly null).</en>
   */
  GradientPaintTransformType getBaseItemFillPaintTransformer() ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼõͼڲķʽ.
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset,and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param transformer GradientPaintTransformType
   * <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The base GradientPaintTransformType object (null permitted).</en>
   */
  void setBaseItemFillPaintTransformer(GradientPaintTransformType transformer) ;

  /**
   * <zh>Ƿͼα߿.</zh>
   * <en>Returns true if an item border is visible,and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>Ʊ߽򷵻true,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  boolean isItemDrawBorder(int row, int column) ;

  /**
   * <zh>Ƿͼα߿,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the visibility of an item border and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setItemDrawBorder(int row, int column, boolean visible) ;

  /**
   * <zh>ݼеһݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset column .</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>Ʊ߽򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isColumnItemDrawBorder(int column) ;

  /**
   * <zh>ݼеһǷƱ߿,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset column drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setColumnItemDrawBorder(int column, boolean visible) ;

  /**
   * <zh>ݼеһݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset row .</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>Ʊ߽򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isRowItemDrawBorder(int row) ;

  /**
   * <zh>ݼеһǷƱ߿,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset row drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setRowItemDrawBorder(int row, boolean visible) ;

  /**
   * <zh>ݼеݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset.</en>
   *
   * @return boolean <zh>Ʊ߽򷵻true,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  boolean isBaseItemDrawBorder() ;

  /**
   * <zh>ݼеǷƱ߿,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setBaseItemDrawBorder(boolean visible) ;

  /**
   * <zh>ڻͼα߿paint.</zh>
   * <en>Returns the paint used to outline data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getItemBorderPaint(int row, int column) ;

  /**
   * <zh>ڻͼα߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline data items as they are drawn and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  void setItemBorderPaint(int row, int column, Paint paint) ;

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the paint used to outline all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  Paint getColumnItemBorderPaint(int column) ;

  /**
   * <zh>ݼеһõı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline all items of a dataset column drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  void setColumnItemBorderPaint(int column, Paint paint) ;

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the paint used to outline all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  Paint getRowItemBorderPaint(int row) ;

  /**
   * <zh>ݼеһõı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline all items of a dataset row drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  void setRowItemBorderPaint(int row, Paint paint) ;

  /**
   * <zh>õı߿paint.</zh>
   * <en>Returns the base paint to outline all items of a dataset.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getBaseItemBorderPaint() ;

  /**
   * <zh>õı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base border paint and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  void setBaseItemBorderPaint(Paint paint) ;

  /**
   * <zh>ڻͼα߿stroke.</zh>
   * <en>Returns the stroke used to outline data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  Stroke getItemBorderStroke(int row, int column) ;

  /**
   * <zh>ڻͼα߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline data items as they are drawn and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (null permitted).</en>
   */
  void setItemBorderStroke(int row, int column, Stroke stroke) ;

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the stroke used to outline all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  Stroke getColumnItemBorderStroke(int column) ;

  /**
   * <zh>ݼеһõı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline all items of a dataset column drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  void setColumnItemBorderStroke(int column, Stroke stroke) ;

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the stroke used to outline all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  Stroke getRowItemBorderStroke(int row) ;

  /**
   * <zh>ݼеһõı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline all items of a dataset row drawn by the render
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  void setRowItemBorderStroke(int row, Stroke stroke) ;

  /**
   * <zh>õı߿paint.</zh>
   * <en>Returns the base stroke to outline all items of a dataset.</en>
   *
   * @return Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  Stroke getBaseItemBorderStroke() ;

  /**
   * <zh>õı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base border stroke and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  void setBaseItemBorderStroke(Stroke stroke) ;

  /**
   * <zh>ĳıǩǷʾ.</zh>
   * <en>Returns true if an item label is visible,and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return <zh>ɼtrue,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  boolean isItemLabelVisible(int row, int column) ;

  /**
   * <zh>ĳıǩǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the visibility of an item label and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setItemLabelVisible(int row, int column, boolean visible) ;

  /**
   * <zh>ݼеһݵıǩǷʾ.</zh>
   * <en>Returns true if the item labels for a column are visible,
   * and false otherwise.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ʾǩ򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isColumnItemLabelVisible(int column) ;

  /**
   * <zh>ݼеһݵıǩǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that controls the visibility of the item labels for a column and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setColumnItemLabelVisible(int column, boolean visible) ;

  /**
   * <zh>ݼеһݵıǩǷʾ.</zh>
   * <en>Returns true if the item labels for a row are visible,
   * and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ʾǩ򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  Boolean isRowItemLabelVisible(int row) ;

  /**
   * <zh>ݼеһݵıǩǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that controls the visibility of the item labels for a row and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  void setRowItemLabelVisible(int row, boolean visible) ;

  /**
   * <zh>ݼеݵıǩǷʾ.</zh>
   * <en>Returns the base setting for item label visibility.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A flag (possibly null).</en>
   */
  boolean getBaseItemLabelVisible() ;

  /**
   * <zh>ݼеݵıǩǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base flag that controls whether or not item labels are visible and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param visible <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  void setBaseItemLabelVisible(boolean visible) ;

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the font for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  Font getItemLabelFont(int row, int column) ;

  /**
   * <zh>ñǩ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the font for an item label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  void setItemLabelFont(int row, int column, Font font) ;

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Returns the font for all items label of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (maybe null).</en>
   */
  Font getColumnItemLabelFont(int column) ;

  /**
   * <zh>ݼһݵıǩ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  void setColumnItemLabelFont(int column, Font font) ;

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Returns the font for all items label of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (maybe null).</en>
   */
  Font getRowItemLabelFont(int row) ;

  /**
   * <zh>ݼһݵıǩ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  void setRowItemLabelFont(int row, Font font) ;

  /**
   * <zh>ݼݵıǩ.</zh>
   * <en>Returns the label font for all items of a dataset.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  Font getBaseItemLabelFont() ;

  /**
   * <zh>ݼݵıǩ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  void setBaseItemLabelFont(Font font) ;

  /**
   * <zh>رǩpaint.</zh>
   * <en>Returns the paint for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getItemLabelPaint(int row, int column) ;

  /**
   * <zh>ñǩpaint,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint for an item label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  void setItemLabelPaint(int row, int column, Paint paint) ;

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  Paint getColumnItemLabelPaint(int column) ;

  /**
   * <zh>ݼһݵıǩpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  void setColumnItemLabelPaint(int column, Paint paint) ;

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  Paint getRowItemLabelPaint(int row) ;

  /**
   * <zh>ݼһݵıǩpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  void setRowItemLabelPaint(int row, Paint paint) ;

  /**
   * <zh>ݼݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  Paint getBaseItemLabelPaint() ;

  /**
   * <zh>ݼݵıǩpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  void setBaseItemLabelPaint(Paint paint) ;

  /**
   * <zh>رǩλ.</zh>
   * <en>Returns the label position.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  TextPosition getItemLabelPosition(int row, int column) ;

  /**
   * <zh>ñǩλ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label position and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  void setItemLabelPosition(int row, int column, TextPosition position) ;

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  TextPosition getColumnItemLabelPosition(int column) ;

  /**
   * <zh>ݼһݵıǩλ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  void setColumnItemLabelPosition(int column, TextPosition position) ;

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  TextPosition getRowItemLabelPosition(int row) ;

  /**
   * <zh>ݼһݵıǩλ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  void setRowItemLabelPosition(int row, TextPosition position) ;

  /**
   * <zh>ݼݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  TextPosition getBaseItemLabelPosition() ;

  /**
   * <zh>ݼݵıǩλ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null not permitted).</en>
   */
  void setBaseItemLabelPosition(TextPosition position) ;

  /**
   * <zh>رǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (never null).</en>
   */
  HorizontalAlignment getItemLabelHorizontalAlignment(int row, int column) ;

  /**
   * <zh>ñǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for an item label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  void setItemLabelHorizontalAlignment(int row, int column, HorizontalAlignment align) ;

  /**
   * <zh>ݼеһݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all items label of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  HorizontalAlignment getColumnItemLabelHorizontalAlignment(int column) ;

  /**
   * <zh>ݼһݵıǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all values items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  void setColumnItemLabelHorizontalAlignment(int column, HorizontalAlignment align) ;

  /**
   * <zh>ݼеһݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all items label of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  HorizontalAlignment getRowItemLabelHorizontalAlignment(int row) ;

  /**
   * <zh>ݼһݵıǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  void setRowItemLabelHorizontalAlignment(int row, HorizontalAlignment align) ;

  /**
   * <zh>ݼеֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all negative value items label of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  HorizontalAlignment getNegativeItemLabelHorizontalAlignment() ;

  /**
   * <zh>ݼֵΪıǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all negative value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  void setNegativeItemLabelHorizontalAlignment(HorizontalAlignment align) ;

  /**
   * <zh>ݼеֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all positive value items label of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  HorizontalAlignment getPositiveItemLabelHorizontalAlignment() ;

  /**
   * <zh>ݼֵΪıǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all positive value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  void setPositiveItemLabelHorizontalAlignment(HorizontalAlignment align) ;

  /**
   * <zh>ݼݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the label horizontal alignment for all items of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (never null).</en>
   */
  HorizontalAlignment getBaseItemLabelHorizontalAlignment() ;

  /**
   * <zh>ݼݵıǩˮƽ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label horizontal alignment for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null not permitted).</en>
   */
  void setBaseItemLabelHorizontalAlignment(HorizontalAlignment align) ;

  /**
   * <zh>رǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (never null).</en>
   */
  VerticalAlignment getItemLabelVerticalAlignment(int row, int column) ;

  /**
   * <zh>ñǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for an item label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  void setItemLabelVerticalAlignment(int row, int column, VerticalAlignment align) ;

  /**
   * <zh>ݼеһݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  VerticalAlignment getColumnItemLabelVerticalAlignment(int column) ;

  /**
   * <zh>ݼһݵıǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all values items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  void setColumnItemLabelVerticalAlignment(int column, VerticalAlignment align) ;

  /**
   * <zh>ݼеһݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  VerticalAlignment getRowItemLabelVerticalAlignment(int row) ;

  /**
   * <zh>ݼһݵıǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  void setRowItemLabelVerticalAlignment(int row, VerticalAlignment align) ;

  /**
   * <zh>ݼеֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for all negative value items label of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  VerticalAlignment getNegativeItemLabelVerticalAlignment() ;

  /**
   * <zh>ݼֵΪıǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all negative value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  void setNegativeItemLabelVerticalAlignment(VerticalAlignment align) ;

  /**
   * <zh>ݼеֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for all positive value items label of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  VerticalAlignment getPositiveItemLabelVerticalAlignment() ;

  /**
   * <zh>ݼֵΪıǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all positive value items label and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  void setPositiveItemLabelVerticalAlignment(VerticalAlignment align) ;

  /**
   * <zh>ݼݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (never null).</en>
   */
  VerticalAlignment getBaseItemLabelVerticalAlignment() ;

  /**
   * <zh>ݼݵıǩĴֱ뷽ʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the label vertical alignment for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null not permitted).</en>
   */
  void setBaseItemLabelVerticalAlignment(VerticalAlignment align) ;

  /**
   * <zh>ͼαǩ{@link ILabelGenerator}.</zh>
   * <en>Returns the item label generator for all items of a dataset.</en>
   *
   * @return ILabelGenerator <zh>ǩɶ (Ϊnull).</zh>
   * <en>The generator (possibly null).</en>
   */
  ILabelGenerator getLabelGenerator() ;

  /**
   * <zh>ͼαǩ{@link ILabelGenerator},
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼</zh>
   * <en>Sets the item label generator for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param generator ILabelGenerator <zh>ǩɶ (Ϊnull).</zh>
   * <en>The generator (null permitted).</en>
   */
  void setLabelGenerator(ILabelGenerator generator) ;

  /**
   * <zh>ͼεtooltip{@link IToolTipGenerator}.</zh>
   * <en>Returns the tool tip generator that should be used for all items of a dataset.</en>
   *
   * @return IToolTipGenerator <zh>tooltipɶ (Ϊnull).</zh>
   * <en>The generator (possibly null).</en>
   */
  IToolTipGenerator getToolTipGenerator() ;

  /**
   * <zh>ͼεtooltip{@link IToolTipGenerator},
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the tool tip generator that should be used for all items of a dataset and
   * sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param generator IToolTipGenerator <zh>tooltipɶ (Ϊnull).</zh>
   * <en>The generator (null permitted).</en>
   */
  void setToolTipGenerator(IToolTipGenerator generator) ;

  /**
   * <zh>Զͼ.</zh>
   * <en>Sets the custom legend type.</en>
   *
   * @param legendType LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  void setCustomLegendType(LegendType legendType) ;

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  LegendType getLegendType() ;

  /**
   * <zh>Ʊɫ.</zh>
   * <en>Draws a background for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼƷ.</zh>
   * <en>The chart draw orientation.</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  void drawBackground(Rectangle2D dataArea, ChartOrientation orientation,
                      RenderShapeCollection renderShapes) ;

  /**
   * <zh>ƱͼƬ.</zh>
   * <en>Draws a background image for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  void drawBackgroundImage(Rectangle2D dataArea, RenderShapeCollection renderShapes) ;

  /**
   * <zh>Ʊ߿.</zh>
   * <en>Draws a border for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  void drawDrawAreaBorder(Rectangle2D dataArea, RenderShapeCollection renderShapes) ;

  /**
   * <zh>עrender仯ļ߶.</zh>
   * <en>Registers an object for notification of changes to the render object.</en>
   *
   * @param listener IRenderChangeListener <zh> (Ϊnull).</zh>
   * <en>The listener (null not permitted).</en>
   */
  void addChangeListener(IRenderChangeListener listener) ;

  /**
   * <zh>ɾעļ.</zh>
   * <en>Removes a change listener.</en>
   *
   * @param listener IRenderChangeListener <zh> (Ϊnull).</zh>
   * <en>The listener (null not permitted).</en>
   */
  void removeChangeListener(IRenderChangeListener listener) ;

  /**
   * <zh>еļ.</zh>
   * <en>Deregisters all object for notification of changes to the render.</en>
   */
  void removeAllChangeListener() ;

}
