package com.ijchart.xychart.render.category ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.MinMaxChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ˮƽֱ͵ͼ.</zh>
 * <en> A render that draws individual data items as horizontal or vertical lines, spaced in the
 * same way as bars in a bar chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MinMaxRender extends AbstractCategoryRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6215052698718246182L ;

  /**
   * <zh>ȱʡǷ.</zh>
   * <en>A default boolean flag controls whether or not draw plot lines.</en>
   */
  public static final boolean DEFAULT_DRAW_GROUP_LINES = false ;

  /**
   * <zh>ȱʡĻߵpaint.</zh>
   * <en>The default paint used to draw group line.</en>
   */
  public static final Paint DEFAULT_GROUP_LINE_PAINT = Color.BLACK ;

  /**
   * <zh>ȱʡĻߵstroke.</zh>
   * <en>The default stroke used to draw group line.</en>
   */
  public static final Stroke DEFAULT_GROUP_LINE_STROKE = new BasicStroke(1.0F) ;

  /**
   * <zh>ȱʡСֵӦͼ.</zh>
   * <en>The default minimum value shape.</en>
   */
  public static final Shape DEFAULT_MIN_SHAPE = new Ellipse2D.Double( -4, -4, 8, 8) ;

  /**
   * <zh>ȱʡĴֵӦͼ.</zh>
   * <en>The default maximum value shape.</en>
   */
  public static final Shape DEFAULT_MAX_SHAPE = new Ellipse2D.Double( -4, -4, 8, 8) ;

  /**
   * <zh>Ƿ.</zh>
   * <en>A boolean flag controls whether or not draw plot lines.</en>
   */
  private boolean drawPlotLine ;

  /**
   * <zh>ߵpaint.</zh>
   * <en>The paint used to draw group line.</en>
   */
  private transient Paint groupLinePaint ;

  /**
   * <zh>ߵstroke.</zh>
   * <en>The stroke used to draw group line.</en>
   */
  private transient Stroke groupLineStroke ;

  /**
   * <zh>СֵӦͼ.</zh>
   * <en>The minimum value shape.</en>
   */
  private transient Shape minShape ;

  /**
   * <zh>ֵӦͼ.</zh>
   * <en>The maximum value shape.</en>
   */
  private transient Shape maxShape ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public MinMaxRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.drawPlotLine = DEFAULT_DRAW_GROUP_LINES ;
    this.groupLinePaint = DEFAULT_GROUP_LINE_PAINT ;
    this.groupLineStroke = DEFAULT_GROUP_LINE_STROKE ;
    this.minShape = DEFAULT_MIN_SHAPE ;
    this.maxShape = DEFAULT_MAX_SHAPE ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Returns a boolean flag controls whether or not draw plot lines.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawPlotLine() {
    return this.drawPlotLine ;
  }

  /**
   * <zh>Ƿ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a boolean flag controls whether or not draw plot lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawPlotLine(boolean flag) {
    if (this.drawPlotLine != flag) {
      this.drawPlotLine = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػߵpaint.</zh>
   * <en>Returns the paint used to draw group line.</en>
   *
   * @return Paint <zh>ߵpaint (Ϊnull).</zh>
   * <en>The paint used to draw group line (maybe null).</en>
   */
  public Paint getGroupLinePaint() {
    return this.groupLinePaint ;
  }

  /**
   * <zh>ûߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw group line and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>ߵpaint (Ϊnull).</zh>
   * <en>The paint used to draw group line (null permitted).</en>
   */
  public void setGroupLinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.groupLinePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػߵstroke.</zh>
   * <en>Returns the stroke used to draw group line.</en>
   *
   * @return Stroke <zh>ߵstroke (Ϊnull).</zh>
   * <en>The stroke used to draw group line (maybe null).</en>
   */
  public Stroke getGroupLineStroke() {
    return this.groupLineStroke ;
  }

  /**
   * <zh>ûߵstroke,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw group line and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>ߵstroke (Ϊnull).</zh>
   * <en>The stroke used to draw group line (null permitted).</en>
   */
  public void setGroupLineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.groupLineStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>СֵӦͼ.</zh>
   * <en>Returns the minimum value shape.</en>
   *
   * @return Shape <zh>СֵӦͼ (Ϊnull).</zh>
   * <en>The minimum value shape (maybe null).</en>
   */
  public Shape getMinShape() {
    return this.minShape ;
  }

  /**
   * <zh>СֵӦͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the minimum value shape and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param shape Shape <zh>СֵӦͼ (Ϊnull).</zh>
   * <en>The minimum value shape (null permitted).</en>
   */
  public void setMinShape(Shape shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument") ;
    }

    this.minShape = shape ;
    fireRenderChanged() ;
  }

  /**
   * <zh>شֵӦͼ.</zh>
   * <en>Returns the maximum value shape.</en>
   *
   * @return Shape <zh>ֵӦͼ (Ϊnull).</zh>
   * <en>The maximum value shape (maybe null).</en>
   */
  public Shape getMaxShape() {
    return this.maxShape ;
  }

  /**
   * <zh>ôֵӦͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the maximum value shape and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param shape Shape <zh>ֵӦͼ (Ϊnull).</zh>
   * <en>The maximum value shape (null permitted).</en>
   */
  public void setMaxShape(Shape shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument") ;
    }

    this.maxShape = shape ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(columns) ;
    getBorderPaints(columns) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(columns) ;

    double lineLength = Math.min(this.minShape.getBounds2D().getWidth(),
                                 this.maxShape.getBounds2D().getHeight()) ;

    //״ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double gapSize = 0 ; //С
      if (columns > 1) {
        gapSize = dataArea.getWidth() * categoryAxis.getCategoryMargin() / (columns - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((columns == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - axisMargin) / columns ; //ÿϵеĿ
      double reallyBarWidth = Math.min(categoryWidth, lineLength) ;
      double halfReallyLineLen = reallyBarWidth * 0.5 ;

      for (int i = 0 ; i < rows ; i++) {
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;
        //Сֵֵͼ
        if (rows > 1 && i == 0) {
          for (int m = 0 ; m < columns ; m++) {
            double[] minMax = calculateDatasetColumnMinMaxValue(dataset, rows, m) ;
            if (Double.isNaN(minMax[0]) || Double.isNaN(minMax[1])) {
              renderShapes.addMinMaxShape(null) ;
            } else {
              double minPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, minMax[0], false) ;
              double maxPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, minMax[1], false) ;
              if (minPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
                  || maxPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                renderShapes.addMinMaxShape(null) ;
                continue ;
              }

              double x11 = startX + (m + 0.5) * categoryWidth + m * gapSize ;
              Point2D minShapePoint = new Point2D.Double(x11, minPos) ;
              Point2D maxShapePoint = new Point2D.Double(x11, maxPos) ;
              MinMaxChartShape minMaxShape = new MinMaxChartShape(minShapePoint, maxShapePoint) ;
              minMaxShape.setMinShape(this.minShape) ;
              minMaxShape.setMaxShape(this.maxShape) ;
              minMaxShape.setGroupLinePaint(this.groupLinePaint) ;
              minMaxShape.setGroupLineStroke(this.groupLineStroke) ;
              renderShapes.addMinMaxShape(minMaxShape) ;
            }
          }
        }

        double lastXPosition = Double.NaN, lastYPosition = Double.NaN ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double y = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              lastXPosition = Double.NaN ;
              lastYPosition = Double.NaN ;
              continue ;
            }

            double x11 = startX + (j + 0.5) * categoryWidth + j * gapSize ;
            LineChartShape lineShape = new LineChartShape(x11 - halfReallyLineLen, y,
                                                          x11 + halfReallyLineLen, y) ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, lineShape) ;
            renderShapes.addDataItemShape(shape) ;

            if (this.drawPlotLine && j > 0 && !Double.isNaN(lastXPosition)) {
              LineChartShape twoValueLine = new LineChartShape(lastXPosition, lastYPosition, x11, y) ;
              twoValueLine.setBorderPaint(this.fillPaints[i]) ;
              twoValueLine.setFillPaint(this.fillPaints[i]) ;
              renderShapes.addMinMaxValueLineShape(twoValueLine) ;
            }

            lastXPosition = x11 ;
            lastYPosition = y ;
          } else {
            lastXPosition = Double.NaN ;
            lastYPosition = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double gapSize = 0 ; //С
      if (columns > 1) {
        gapSize = dataArea.getHeight() * categoryAxis.getCategoryMargin() / (columns - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((columns == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - axisMargin) / columns ; //ÿϵеĿ
      double reallyBarWidth = Math.min(categoryWidth, lineLength) ;
      double halfReallyLineLen = reallyBarWidth * 0.5 ;

      for (int i = 0 ; i < rows ; i++) {
        double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;
        //Сֵֵͼ
        if (rows > 1 && i == 0) {
          for (int m = 0 ; m < columns ; m++) {
            double[] minMax = calculateDatasetColumnMinMaxValue(dataset, rows, m) ;
            if (Double.isNaN(minMax[0]) || Double.isNaN(minMax[1])) {
              renderShapes.addMinMaxShape(null) ;
            } else {
              double minPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, minMax[0], false) ;
              double maxPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, minMax[1], false) ;
              if (minPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
                  || maxPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                renderShapes.addMinMaxShape(null) ;
                continue ;
              }

              double y11 = startY + (m + 0.5) * categoryWidth + m * gapSize ;
              Point2D minShapePoint = new Point2D.Double(minPos, y11) ;
              Point2D maxShapePoint = new Point2D.Double(maxPos, y11) ;
              MinMaxChartShape minMaxShape = new MinMaxChartShape(minShapePoint, maxShapePoint) ;
              minMaxShape.setMinShape(this.minShape) ;
              minMaxShape.setMaxShape(this.maxShape) ;
              minMaxShape.setGroupLinePaint(this.groupLinePaint) ;
              minMaxShape.setGroupLineStroke(this.groupLineStroke) ;
              renderShapes.addMinMaxShape(minMaxShape) ;
            }
          }
        }

        double lastXPosition = Double.NaN, lastYPosition = Double.NaN ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double x = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              lastXPosition = Double.NaN ;
              lastYPosition = Double.NaN ;
              continue ;
            }

            double y11 = startY + (j + 0.5) * categoryWidth + j * gapSize ;
            LineChartShape lineShape = new LineChartShape(x, y11 - halfReallyLineLen,
                                                          x, y11 + halfReallyLineLen) ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, lineShape) ;
            renderShapes.addDataItemShape(shape) ;

            if (this.drawPlotLine && j > 0 && !Double.isNaN(lastXPosition)) {
              LineChartShape twoValueLine = new LineChartShape(lastXPosition, lastYPosition, x, y11) ;
              twoValueLine.setBorderPaint(this.fillPaints[i]) ;
              twoValueLine.setFillPaint(this.fillPaints[i]) ;
              renderShapes.addMinMaxValueLineShape(twoValueLine) ;
            }

            lastXPosition = x ;
            lastYPosition = y11 ;
          } else {
            lastXPosition = Double.NaN ;
            lastYPosition = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }

  }

  /**
   * <zh>ݼָеСֵֵ.</zh>
   * <en>Calculate the minimum and maximum value \u200B\u200Bof dataset specified column.</en>
   * @param dataset IMatrixDataset
   * @param rowCount int
   * @param column int
   * @return double[]
   */
  private static double[] calculateDatasetColumnMinMaxValue(IMatrixDataset dataset, int rowCount,
                                                            int column) {
    double min = Double.NaN, max = Double.NaN ;
    for (int i = 0 ; i < rowCount ; i++) {
      Object data = dataset.getData(i, column) ;
      if (data != null && data instanceof NumberChartData) {
        NumberChartData num = (NumberChartData) data ;
        double value = num.getValue() ;
        if (Double.isNaN(value)) {
          continue ;
        }

        if (Double.isNaN(min)) {
          min = value ;
        } else {
          min = Math.min(min, value) ;
        }

        if (Double.isNaN(max)) {
          max = value ;
        } else {
          max = Math.max(max, value) ;
        }
      }
    }

    return new double[] {min, max} ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof MinMaxRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    MinMaxRender that = (MinMaxRender) obj ;
    if (this.drawPlotLine != that.drawPlotLine) {
      return false ;
    }
    if (!EqualsUtil.equals(this.groupLinePaint, that.groupLinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.groupLineStroke, that.groupLineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.minShape, that.minShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.maxShape, that.maxShape)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.drawPlotLine) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.groupLinePaint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.groupLineStroke) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.minShape) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.maxShape) ;

    return hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "MinMaxRender" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.groupLinePaint) ;
    SerialUtil.writeStroke(stream, this.groupLineStroke) ;
    SerialUtil.writeShape(stream, this.minShape) ;
    SerialUtil.writeShape(stream, this.maxShape) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.groupLinePaint = SerialUtil.readPaint(stream) ;
    this.groupLineStroke = SerialUtil.readStroke(stream) ;
    this.minShape = SerialUtil.readShape(stream) ;
    this.maxShape = SerialUtil.readShape(stream) ;
  }

}
