package com.ijchart.xychart.render.xy ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.annotation.IXYAnnotation ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.render.AbstractWithAxisRender ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩxy renderĳʵ.</zh>
 * <en>Base class providing common services for xy renders.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractXYRender extends AbstractWithAxisRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7954551675645972588L ;

  /**
   * <zh>ע(annotation)󼯺.</zh>
   * <en>The annotation collection.</en>
   */
  private List annotations ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  protected AbstractXYRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.annotations = new ArrayList() ;
  }

  /**
   * <zh>һע(annotation),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Add an annotation and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param annotation IXYAnnotation <zh>ע(annotation) (Ϊnull).</zh>
   * <en>A annotation object (null not permitted).</en>
   */
  public void addAnnotation(IXYAnnotation annotation) {
    if (annotation == null) {
      throw new IllegalArgumentException("Null 'annotation' argument.") ;
    }

    this.annotations.add(annotation) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ɾеע(annotation),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Remove all annotations of this render and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   */
  public void removeAllAnnotation() {
    this.annotations.clear() ;
  }

  /**
   * <zh>еע(annotation),˼ǲܱĵ.</zh>
   * <en>Returns all annotations and this collection is unmodifiable.</en>
   *
   * @return Collection <zh>ע(annotation)󼯺 (Ϊnull).</zh>
   * <en>The collection of annotations (never null).</en>
   */
  public Collection getAllAnnotations() {
    return Collections.unmodifiableCollection(this.annotations) ;
  }

  /**
   * <zh>еע(annotation)ͼ.</zh>
   * <en>Draws all annotation items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>x (Ϊnull).</zh>
   * <en>The x-axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ (Ϊnull).</zh>
   * <en>The x-axis edge (null not permitted).</en>
   * @param yAxis AbstractValueAxis <zh>y (Ϊnull).</zh>
   * <en>The y-axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y-axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAllAnnotation(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                                AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                                Rectangle2D dataArea,
                                ChartOrientation orientation,
                                AnnotationShapeCollection annotationShapes) {
    for (int i = 0 ; i < this.annotations.size() ; i++) {
      IXYAnnotation annotation = (IXYAnnotation)this.annotations.get(i) ;
      if (annotation != null) {
        annotation.drawAnnotation(xAxis, xAxisEdge, yAxis, yAxisEdge,
                                  dataArea, orientation, annotationShapes) ;
      }
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public abstract void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                                    AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                                    Rectangle2D dataArea, ChartOrientation orientation,
                                    RenderShapeCollection renderShapes, float alpha) ;

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractXYRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractXYRender that = (AbstractXYRender) obj ;
    if (EqualsUtil.equals(this.annotations, that.annotations)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.annotations) ;

    return hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractXYRender" ;
  }

}
