package com.ijchart.xychart.render.xy ;

import java.awt.Paint ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.OHLCChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.shape.HighLowChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>high-lowͼ.</zh>
 * <en>Draw high-low chart shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class HighLowRender extends AbstractXYRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -439407848678868990L ;

  /**
   * <zh>ǷopenֵӦֱߵȱʡֵ.</zh>
   * <en>A default flag that controls whether the open line is drawn.</en>
   */
  public static final boolean DEFAULT_DRAW_OPEN_LINE = true ;

  /**
   * <zh>ǷcloseֵӦֱߵȱʡֵ.</zh>
   * <en>A default flag that controls whether the close line is drawn.</en>
   */
  public static final boolean DEFAULT_DRAW_CLOSE_LINE = true ;

  /**
   * <zh>ǷopenֵӦֱ.</zh>
   * <en>A flag that controls whether the open line is drawn.</en>
   */
  private boolean drawOpenLine ;

  /**
   * <zh>ǷcloseֵӦֱ.</zh>
   * <en>A flag that controls whether the close line is drawn.</en>
   */
  private boolean drawCloseLine ;

  /**
   * <zh>openֵӦֱߵpaint.</zh>
   * <en>The paint used for the open line.</en>
   */
  private transient Paint openLinePaint ;

  /**
   * <zh>closeֵӦֱߵpaint.</zh>
   * <en>The paint used for the close line.</en>
   */
  private transient Paint closeLinePaint ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public HighLowRender(SingleRowDataset dataset) {
    super(dataset) ;

    this.drawOpenLine = DEFAULT_DRAW_OPEN_LINE ;
    this.drawCloseLine = DEFAULT_DRAW_CLOSE_LINE ;
    this.openLinePaint = null ;
    this.closeLinePaint = null ;
  }

  /**
   * <zh>ǷopenֵӦֱ.</zh>
   * <en>Returns the flag that controls whether the open line is drawn.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawOpenLine() {
    return this.drawOpenLine ;
  }

  /**
   * <zh>ǷopenֵӦֱ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether the open line is drawn and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawOpenLine(boolean flag) {
    if (this.drawOpenLine != flag) {
      this.drawOpenLine = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ǷcloseֵӦֱ.</zh>
   * <en>Returns the flag that controls whether the close line is drawn.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawCloseLine() {
    return this.drawCloseLine ;
  }

  /**
   * <zh>ǷcloseֵӦֱ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether the close line is drawn and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawCloseLine(boolean flag) {
    if (this.drawCloseLine != flag) {
      this.drawCloseLine = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػopenֵӦֱߵpaint.</zh>
   * <en>Returns the paint used for the open line.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getOpenLinePaint() {
    return this.openLinePaint ;
  }

  /**
   * <zh>ûopenֵӦֱߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used for the open line and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setOpenLinePaint(Paint paint) {
    if (!(this.openLinePaint == null && paint == null)) {
      this.openLinePaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػcloseֵӦֱߵpaint.</zh>
   * <en>Returns the paint used for the close line.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getCloseLinePaint() {
    return this.closeLinePaint ;
  }

  /**
   * <zh>ûcloseֵӦֱߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used for the close line and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setCloseLinePaint(Paint paint) {
    if (!(this.closeLinePaint == null && paint == null)) {
      this.closeLinePaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    boolean isVertical = orientation.equals(ChartOrientation.VERTICAL) ; //ֱ

    //stockͼ
    if (dataset.getRowCount() > 0) {
      boolean allIsNull = true ;
      int currRowCols = dataset.getColumnCount(0) ;

      for (int i = 0 ; i < currRowCols ; i++) {
        Object obj = dataset.getData(0, i) ;
        if (obj != null) {
          allIsNull = false ;
          break ;
        }
      }
      if (allIsNull) {
        return ;
      }

      final double delta = 2.0D ; //opencloseֱߵĳ
      Paint openPaint = null, closePaint = null ;
      Paint fillPaint = fillPaints[0] ;
      if (currRowCols > 0) {
        if (this.openLinePaint == null) {
          openPaint = fillPaint ;
        } else {
          openPaint = this.openLinePaint ;
        }
        if (this.closeLinePaint == null) {
          closePaint = fillPaint ;
        } else {
          closePaint = this.closeLinePaint ;
        }
      }

      for (int i = 0 ; i < currRowCols ; i++) {
        OHLCChartData data = (OHLCChartData) dataset.getData(0, i) ;
        if (data != null) {
          long dateMills = data.getDate().getTime() ;
          double open = data.getOpen() ;
          double close = data.getClose() ;
          double high = data.getHigh() ;
          double low = data.getLow() ;

          if (Double.isNaN(open) || Double.isNaN(close)
              || Double.isNaN(high) || Double.isNaN(low)) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          double xPos = xAxis.valueToJava2D(dataArea, xAxisEdge, dateMills, false) ;
          if (xPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          //draw high-low line
          Line2D highLowLine = null ;
          Rectangle2D drawArea = null ;
          if (!Double.isNaN(high) && !Double.isNaN(low)) {
            double highPos = yAxis.valueToJava2D(dataArea, yAxisEdge, high, false) ;
            double lowPos = yAxis.valueToJava2D(dataArea, yAxisEdge, low, false) ;
            if (highPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || lowPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            double minPos = Math.min(highPos, lowPos) ;
            double posLen = Math.abs(highPos - lowPos) ;
            if (isVertical) {
              highLowLine = new Line2D.Double(xPos, highPos, xPos, lowPos) ;
              drawArea = new Rectangle2D.Double(xPos - delta, minPos, delta * 2, posLen) ;
            } else {
              highLowLine = new Line2D.Double(lowPos, xPos, highPos, xPos) ;
              drawArea = new Rectangle2D.Double(minPos, xPos - delta, posLen, delta * 2) ;
            }
          } else {
            if (isVertical) {
              drawArea = new Rectangle2D.Double(xPos - delta, dataArea.getCenterY(), 2 * delta, 1) ;
            } else {
              drawArea = new Rectangle2D.Double(dataArea.getCenterY(), xPos - delta, 1, 2 * delta) ;
            }
          }

          Line2D openLine = null, closeLine = null ;
          if (!Double.isNaN(open)) {
            double openPos = yAxis.valueToJava2D(dataArea, yAxisEdge, open, false) ;
            if (openPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            if (isVertical) {
              openLine = new Line2D.Double(xPos - delta, openPos, xPos, openPos) ;
            } else {
              openLine = new Line2D.Double(openPos, xPos, openPos, xPos + delta) ;
            }
          }

          if (!Double.isNaN(close)) {
            double closePos = yAxis.valueToJava2D(dataArea, yAxisEdge, close, false) ;
            if (closePos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            if (isVertical) {
              closeLine = new Line2D.Double(xPos, closePos, xPos + delta, closePos) ;
            } else {
              closeLine = new Line2D.Double(closePos, xPos - delta, closePos, xPos) ;
            }
          }

          HighLowChartShape stockShape = new HighLowChartShape(drawArea, null, highLowLine,
                                                               fillPaint, openLine, openPaint,
                                                               closeLine, closePaint) ;

          IChartShape chartShape = createItemShape(orientation, yAxis, 0, i, drawArea,
                                                   fillPaints[0], borderPaints[0],
                                                   borderStrokes[0], alpha, stockShape) ;
          renderShapes.addDataItemShape(chartShape) ;
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }

  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof HighLowRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    HighLowRender that = (HighLowRender) obj ;
    if (this.drawOpenLine != that.drawOpenLine) {
      return false ;
    }
    if (this.drawCloseLine != that.drawCloseLine) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openLinePaint, that.openLinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.closeLinePaint, that.closeLinePaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.drawOpenLine) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.drawCloseLine) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.openLinePaint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.closeLinePaint) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   *   java.lang.Cloneable
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "HighLowRender" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.openLinePaint) ;
    SerialUtil.writePaint(stream, this.closeLinePaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.openLinePaint = SerialUtil.readPaint(stream) ;
    this.closeLinePaint = SerialUtil.readPaint(stream) ;
  }

}
