package com.ijchart.xychart.render.xy ;

import java.awt.BasicStroke ;
import java.awt.Paint ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.XYZNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.render.paint.IPaintScale ;
import com.ijchart.xychart.render.paint.LookupPaintScale ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.PaintScaleLegendChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ݼÿ{@link com.ijchart.xychart.data.XYZNumberChartData},(x,y)ֵȷλ,
 * zֵȷɫ.</zh>
 * <en>A render that represents data from a {@link com.ijchart.xychart.data.XYZNumberChartData}
 *  by drawing a color block at each (x, y) point, where the color is a function of the
 * z-value from the dataset.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBlockRender extends AbstractXYRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 903589175244099948L ;

  /**
   * <zh>ȱʡĿĿ.</zh>
   * <en>The default block width.</en>
   */
  public static final double DEFAULT_BLOCK_WIDTH = 1.0 ;

  /**
   * <zh>ȱʡĿĸ߶.</zh>
   * <en>The default block height.</en>
   */
  public static final double DEFAULT_BLOCK_HEIGHT = 1.0 ;

  /**
   * <zh>ȱʡ{@link IPaintScale}Ķ.</zh>
   * <en>The default paint scale.</en>
   */
  public static final IPaintScale DEFAULT_PAINT_SCALE = new LookupPaintScale() ;

  /**
   * <zh>ȱʡͼαǩ.</zh>
   * <en>The legend label.</en>
   */
  public static final String DEFAULT_LEGEND_LABEL = "Legend" ;

  /**
   * <zh>Ŀ.</zh>
   * <en>The block width.</en>
   */
  private double blockWidth ;

  /**
   * <zh>ĸ߶.</zh>
   * <en>The block height.</en>
   */
  private double blockHeight ;

  /**
   * <zh>ʵֽӿ{@link IPaintScale}Ķ.</zh>
   * <en>The paint scale.</en>
   */
  private IPaintScale paintScale ;

  /**
   * <zh>ͼǩ.</zh>
   * <en>The legend label.</en>
   */
  private String legendLabel ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYBlockRender(IMatrixDataset dataset) {
    this(dataset, DEFAULT_PAINT_SCALE, DEFAULT_LEGEND_LABEL) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param legendLabel String <zh>ͼǩ.</zh>
   * <en>The legend label.</en>
   */
  public XYBlockRender(IMatrixDataset dataset, String legendLabel) {
    this(dataset, DEFAULT_PAINT_SCALE, legendLabel) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param paintScale <zh>IPaintScale.</zh>
   * <en>The paint scale.</en>
   * @param legendLabel String <zh>ͼǩ.</zh>
   * <en>The legend label.</en>
   */
  public XYBlockRender(IMatrixDataset dataset, IPaintScale paintScale, String legendLabel) {
    this(dataset, DEFAULT_BLOCK_WIDTH, DEFAULT_BLOCK_HEIGHT, paintScale, legendLabel) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param width <zh>Ŀ.</zh>
   * <en>The block width.</en>
   * @param height <zh>ĸ߶.</zh>
   * <en>The block height.</en>
   * @param paintScale <zh>IPaintScale.</zh>
   * <en>The paint scale.</en>
   * @param legendLabel String <zh>ͼǩ.</zh>
   * <en>The legend label.</en>
   */
  public XYBlockRender(IMatrixDataset dataset, double width, double height,
                       IPaintScale paintScale, String legendLabel) {
    super(dataset) ;
    if (paintScale == null) {
      throw new IllegalArgumentException("Null 'paintScale' argument") ;
    }

    this.blockWidth = width ;
    this.blockHeight = height ;
    this.paintScale = paintScale ;
    this.legendLabel = legendLabel ;
  }

  /**
   * <zh>ؿĿ.</zh>
   * <en>Returns the block width.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  public double getBlockWidth() {
    return this.blockWidth ;
  }

  /**
   * <zh>ÿĿ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the block width and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param width double <zh>.</zh>
   * <en>The width.</en>
   */
  public void setBlockWidth(double width) {
    if (width <= 0.0) {
      throw new IllegalArgumentException("Argument 'width' must be >0.0") ;
    }

    if (this.blockWidth != width) {
      this.blockWidth = width ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؿĸ߶.</zh>
   * <en>Returns the block height.</en>
   *
   * @return double <zh>߶.</zh>
   * <en>The height.</en>
   */
  public double getBlockHeight() {
    return this.blockHeight ;
  }

  /**
   * <zh>ÿĿ,м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the block width and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param height double <zh>߶.</zh>
   * <en>The height.</en>
   */
  public void setBlockHeight(double height) {
    if (height <= 0.0) {
      throw new IllegalArgumentException("Argument 'height' must be >0.0") ;
    }

    if (this.blockHeight != height) {
      this.blockHeight = height ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>IPaintScaleĶ.</zh>
   * <en>Returns the paint scale.</en>
   *
   * @return IPaintScale <zh>IPaintScaleĶ.</zh>
   * <en>The paint scale.</en>
   */
  public IPaintScale getPaintScale() {
    return this.paintScale ;
  }

  /**
   * <zh>IPaintScaleĶ,м{@link com.ijchart.xychart.event.RenderChangeEvent}
   * ¼.</zh>
   * <en>Sets the paint scale and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paintScale IPaintScale <zh>IPaintScaleĶ.</zh>
   * <en>The paint scale.</en>
   */
  public void setPaintScale(IPaintScale paintScale) {
    if (paintScale == null) {
      throw new IllegalArgumentException("Null 'paintScale' argument") ;
    }

    this.paintScale = paintScale ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ͼǩ.</zh>
   * <en>Returns the legend label.</en>
   *
   * @return String <zh>ͼǩ (Ϊnull).</zh>
   * <en>The legend label (maybe null).</en>
   */
  public String getLegendLabel() {
    return this.legendLabel ;
  }

  /**
   * <zh>ͼǩ.</zh>
   * <en>The legend label.</en>
   *
   * @param label String <zh>ͼǩ (Ϊnull).</zh>
   * <en>The legend label (null permitted).</en>
   */
  public void setLegendLabel(String label) {
    if (EqualsUtil.equals(this.legendLabel, label)) {
      this.legendLabel = label ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    double xOffset = -this.blockWidth / 2.0 ;
    double yOffset = -this.blockHeight / 2.0 ;
    int rows = dataset.getRowCount() ;
    BasicStroke borderStroke = new BasicStroke(1.0f) ;

    getFillPaints(rows) ;
    getBorderPaints(rows) ;
    getBorderStrokes(rows) ;

    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof XYZNumberChartData) {
          XYZNumberChartData xyz = (XYZNumberChartData) data ;
          double x = xyz.getX() ;
          double y = xyz.getY() ;
          double z = xyz.getZ() ;
          if (Double.isNaN(x) || Double.isNaN(y) || Double.isNaN(z)) {
            renderShapes.addDataItemShape(null) ;
          } else {
            double xx0 = xAxis.valueToJava2D(dataArea, xAxisEdge, x + xOffset, false) ;
            double xx1 = xAxis.valueToJava2D(dataArea, xAxisEdge,
                                             x + xOffset + this.blockWidth, false) ;
            double yy0 = yAxis.valueToJava2D(dataArea, yAxisEdge, y + yOffset, false) ;
            double yy1 = yAxis.valueToJava2D(dataArea, yAxisEdge,
                                             y + yOffset + this.blockHeight, false) ;
            if (xx0 == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || xx1 == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || yy0 == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || yy1 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            Rectangle2D block = null ;
            if (orientation.equals(ChartOrientation.HORIZONTAL)) {
              block = new Rectangle2D.Double(Math.min(yy0, yy1), Math.min(xx0, xx1),
                                             Math.abs(yy1 - yy0), Math.abs(xx0 - xx1)) ;
            } else {
              block = new Rectangle2D.Double(Math.min(xx0, xx1), Math.min(yy0, yy1),
                                             Math.abs(xx1 - xx0), Math.abs(yy1 - yy0)) ;
            }

            Paint p = this.paintScale.getPaint(z) ;
            RectangleTextChartShape shape = new RectangleTextChartShape(block, null) ;
            shape.setFillShape(true) ;
            shape.setDrawBorder(true) ;

            IChartShape chartShape = createItemShape(orientation, yAxis, i, j, block, p,
                                                     p, borderStroke, alpha, null) ;
            renderShapes.addDataItemShape(chartShape) ;
          }
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }
  }

  /**
   * <zh>ͼĸ.</zh>
   * <en>Returns the count of legends.</en>
   *
   * @return int <zh>ͼĸ.</zh>
   * <en>The count of legends.</en>
   */
  protected int getLegendItemCount() {
    return 1 ;
  }

  /**
   * <zh>ͼͼεıǩ.</zh>
   * <en>Returns the label array for legend item shapes.</en>
   *
   * @return String[] <zh>ǩ</zh>
   * <en>The label array.</en>
   */
  protected String[] getLegendItemLabels() {
    return new String[] {""} ;
  }

  /**
   * <zh>ÿͼͼ.</zh>
   * <en>Returns the shape array of legend items.</en>
   *
   * @param legendCount int <zh>ͼĸ.</zh>
   * <en>The legend items count.</en>
   * @return IChartShape[] <zh>ͼ.</zh>
   * <en>The shapes array.</en>
   */
  protected IChartShape[] getLegendItemShapes(int legendCount) {
    if (legendCount > 0) {
      PaintScaleLegendChartShape shape =
        new PaintScaleLegendChartShape(this.paintScale, this.legendLabel,
                                       getLegendPositon(), getChartArea(), getDataset()) ;
      IChartShape[] shapes = new IChartShape[legendCount] ;
      for (int i = 0 ; i < legendCount ; i++) {
        if (i == 0) {
          shapes[i] = shape ;
        } else {
          shapes[i] = null ;
        }
      }

      return shapes ;
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYBlockRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYBlockRender that = (XYBlockRender) obj ;
    if (this.blockWidth != that.blockWidth) {
      return false ;
    }
    if (this.blockHeight != that.blockHeight) {
      return false ;
    }
    if (!EqualsUtil.equals(this.paintScale, that.paintScale)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.legendLabel, that.legendLabel)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + this.blockWidth ;
    hashcode = 37 * hashcode + this.blockHeight ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.paintScale) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.legendLabel) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   *   java.lang.Cloneable
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[blockWidth=" + this.blockWidth
      + ",blockHeight=" + this.blockHeight
      + ",legendLabel=" + this.legendLabel
      + "]" ;
  }

}
