package com.ijchart.xychart.render.xy ;

import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IXYChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineAndShapeChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.MathUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>֮Ĳͼ.</zh>
 * <en>Draw difference shapes between two rows.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYDifferenceRender extends AbstractXYAreaRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6065692235449239244L ;

  /**
   * <zh>ֵΪֵʱȱʡɫ,(y(0) > y(1)).</zh>
   * <en>The default paint used to highlight positive differences (y(0) > y(1)).</en>
   */
  public static final Paint DEFAULT_POSITIVE_PAINT = Color.GREEN ;

  /**
   * <zh>ֵΪֵʱȱʡɫ,(y(0) < y(1)).</zh>
   * <en>The default paint used to highlight negative differences (y(0) < y(1)).</en>
   */
  public static final Paint DEFAULT_NEGATIVE_PAINT = Color.PINK ;

  /**
   * <zh>ȱʡǷʾͼ.</zh>
   * <en>The default flag whether or not display shapes at each point.</en>
   */
  public static final boolean DEFAULT_SHAPES_VISIBLE = false ;

  /**
   * <zh>ֵΪֵʱɫ,(y(0) > y(1)).</zh>
   * <en>The paint used to highlight positive differences (y(0) > y(1)).</en>
   */
  private transient Paint positivePaint ;

  /**
   * <zh>ֵΪֵʱɫ,(y(0) < y(1)).</zh>
   * <en>The paint used to highlight negative differences (y(0) < y(1)).</en>
   */
  private transient Paint negativePaint ;

  /**
   * <zh>Ƿʾͼ.</zh>
   * <en>Display shapes at each point.</en>
   */
  private boolean shapesVisible ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYDifferenceRender(IMatrixDataset dataset) {
    this(dataset, DEFAULT_POSITIVE_PAINT, DEFAULT_NEGATIVE_PAINT) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param positivePaint Paint <zh>ֵΪֵʱɫ (Ϊnull).</zh>
   * <en>The paint used to highlight positive differences (null not permitted).</en>
   * @param negativePaint Paint <zh>ֵΪֵʱɫ (Ϊnull).</zh>
   * <en>The paint used to highlight negative differences (null not permitted).</en>
   */
  public XYDifferenceRender(IMatrixDataset dataset, Paint positivePaint, Paint negativePaint) {
    super(dataset) ;
    if (positivePaint == null) {
      throw new IllegalArgumentException("Null 'positivePaint' argument") ;
    }
    if (negativePaint == null) {
      throw new IllegalArgumentException("Null 'negativePaint' argument") ;
    }

    this.positivePaint = positivePaint ;
    this.negativePaint = negativePaint ;
    this.shapesVisible = DEFAULT_SHAPES_VISIBLE ;
    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
  }

  /**
   * <zh>زֵΪֵʱɫ,(y(0) > y(1)).</zh>
   * <en>Returns the paint used to highlight positive differences (y(0) > y(1)).</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getPositivePaint() {
    return this.positivePaint ;
  }

  /**
   * <zh>òֵΪֵʱɫ,(y(0) > y(1)),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to highlight positive differences (y(0) > y(1)) and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setPositivePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.positivePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>زֵΪֵʱɫ,(y(0) < y(1)).</zh>
   * <en>Returns the paint used to highlight negative differences (y(0) < y(1)).</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getNegativePaint() {
    return this.negativePaint ;
  }

  /**
   * <zh>òֵΪֵʱɫ,(y(0) < y(1)),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to highlight negative differences (y(0) < y(1)) and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setNegativePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.negativePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>Ƿʾͼ.</zh>
   * <en>Returns whether or not display shapes at each point.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isShapesVisible() {
    return this.shapesVisible ;
  }

  /**
   * <zh>Ƿʾͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not display shapes at each point and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setShapesVisible(boolean flag) {
    if (this.shapesVisible != flag) {
      this.shapesVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    int cols = dataset.getColumnCount() ;
    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    double lastX = Double.NaN ; //һXֵ
    double lastY = Double.NaN ; //һYֵ

    Point2D[][] coordinate = new Point2D[rows][cols] ;
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < rows ; i++) {
      Shape shape = this.shapesVisible ? supplier.getNextShape() : null ;
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof IXYChartData) {
          double xValue = ((IXYChartData) data).getX() ;
          double x = xAxis.valueToJava2D(dataArea, xAxisEdge, xValue, false) ;
          double yValue = ((IXYChartData) data).getY() ;
          double y = yAxis.valueToJava2D(dataArea, yAxisEdge, yValue, false) ;

          if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ֱ
            double temp = y ;
            y = x ;
            x = temp ;
          }

          //ֵ᷶Χֵ
          if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            lastX = Double.NaN ;
            lastY = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          Line2D line = null ;
          if (j == 0) {
            line = null ;
          } else {
            if (!Double.isNaN(lastX) && !Double.isNaN(lastY)) {
              line = new Line2D.Double(lastX - x, lastY - y, 0, 0) ;
            }
          }

          coordinate[i][j] = new Point2D.Double(x, y) ;
          LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(x, y, shape, line) ;
          IChartShape chartShape = createItemShape(orientation, yAxis, i, j, null,
                                                   fillPaints[i], borderPaints[i],
                                                   borderStrokes[i], alpha, lineChartShape) ;
          renderShapes.addDataItemShape(chartShape) ;

          lastX = x ;
          lastY = y ;
        } else {
          lastX = Double.NaN ;
          lastY = Double.NaN ;
          renderShapes.addDataItemShape(null) ;
        }
      }
    }

    for (int i = 0 ; i < rows ; i += 2) {
      int currRowCols = dataset.getColumnCount(i) ;
      if (i + 1 <= rows - 1) { //has next row
        int nextRowCols = dataset.getColumnCount(i + 1) ;
        int maxRowCols = Math.max(currRowCols, nextRowCols) ;

        int startCol = 0, endCol = 0 ; //start and end column index
        int lastSign = -2, currSign = -2 ;
        boolean needDraw = false ;
        boolean isNull = false ;
        boolean isVertical = orientation.equals(ChartOrientation.VERTICAL) ;
        for (int j = 0 ; j < maxRowCols ; j++) {
          Point2D currRowPoint = coordinate[i][j] ;
          Point2D nextRowPoint = coordinate[i + 1][j] ;
          isNull = false ;
          if (currRowPoint == null || nextRowPoint == null) {
            endCol = j - 1 ;
            isNull = true ;
            needDraw = true ;
          } else if (j == maxRowCols - 1) {
            endCol = j ;
            isNull = false ;
            needDraw = true ;
          }

          if (!needDraw) {
            if (isVertical) {
              currSign = MathUtil.sign(currRowPoint.getY(), nextRowPoint.getY()) ;
            } else {
              currSign = MathUtil.sign(currRowPoint.getX(), nextRowPoint.getX()) ;
            }

            if (currSign == 0) {
              continue ;
            }

            if (lastSign == -2) {
              lastSign = currSign ;
            } else {
              if (lastSign != currSign) {
                endCol = j - 1 ;
                needDraw = true ;
              }
            }
          }

          if (j == maxRowCols - 1) {
            needDraw = true ;
            endCol = maxRowCols - 1 ;
          }

          if (needDraw && lastSign != -2 && lastSign != 0 && endCol > startCol) {
            GeneralPath path = new GeneralPath() ;
            for (int m = startCol ; m <= endCol ; m++) {
              Point2D currPoint = coordinate[i][m] ;
              if (m == startCol) {
                path.moveTo((float) currPoint.getX(), (float) currPoint.getY()) ;
              } else {
                path.lineTo((float) currPoint.getX(), (float) currPoint.getY()) ;
              }
            }

            for (int m = endCol ; m >= startCol ; m--) {
              Point2D currPoint = coordinate[i + 1][m] ;
              path.lineTo((float) currPoint.getX(), (float) currPoint.getY()) ;
            }

            path.closePath() ;
            GeneralPathChartShape pathShape = new GeneralPathChartShape(path) ;
            Paint fillPaint = null ;
            if (isVertical) {
              if (yAxis.isInverted()) {
                if (lastSign == 1) {
                  fillPaint = this.positivePaint ;
                } else {
                  fillPaint = this.negativePaint ;
                }
              } else {
                if (lastSign == 1) {
                  fillPaint = this.negativePaint ;
                } else {
                  fillPaint = this.positivePaint ;
                }
              }
            } else {
              if (yAxis.isInverted()) {
                if (lastSign == 1) {
                  fillPaint = this.positivePaint ;
                } else {
                  fillPaint = this.negativePaint ;
                }
              } else {
                if (lastSign == 1) {
                  fillPaint = this.negativePaint ;
                } else {
                  fillPaint = this.positivePaint ;
                }
              }
            }

            pathShape.setFillShape(true) ;
            pathShape.setFillPaint(fillPaint) ;
            pathShape.setFillAlpha(alpha) ;
            pathShape.setDrawBorder(false) ;
            renderShapes.addDifferenceBackgroundShapes(pathShape) ;

            startCol = j ;
          }

          if (!isNull && needDraw) {
            lastSign = currSign ;
          }
          needDraw = false ;
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ÿͼͼ.</zh>
   * <en>Returns the shape array of legend items.</en>
   *
   * @param legendCount int <zh>ͼĸ.</zh>
   * <en>The legend items count.</en>
   * @return IChartShape[] <zh>ͼ.</zh>
   * <en>The shapes array.</en>
   */
  protected IChartShape[] getLegendItemShapes(int legendCount) {
    if (legendCount > 0) {
      IDrawingSupplier supplier = getDrawingSupplier() ;
      IChartShape[] shapes = new IChartShape[legendCount] ;
      for (int i = 0 ; i < legendCount ; i++) {
        Shape shape = supplier.getNextShape() ;
        if (!this.shapesVisible) {
          shape = LineAndShapeChartShape.ZERO_RECT ;
        }

        Rectangle2D bound = shape.getBounds() ;
        Line2D line = new Line2D.Double(bound.getMinX() - 6, 0, bound.getMaxX() + 6, 0) ;

        LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(0, 0, shape, line) ;
        lineChartShape.setDrawBorder(true) ;
        lineChartShape.setFillShape(isItemFill(i, 0)) ;
        lineChartShape.setGradientPaintTransformType(getItemFillPaintTransformer(i, 0)) ;

        if (!isAutoSelectBorderPaint()) {
          lineChartShape.setBorderPaint(getItemBorderPaint(i, 0)) ;
        }

        if (!isAutoSelectBorderStroke()) {
          lineChartShape.setBorderStroke(getItemBorderStroke(i, 0)) ;
        }

        shapes[i] = lineChartShape ;
      }

      return shapes ;
    }

    return null ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYDifferenceRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYDifferenceRender that = (XYDifferenceRender) obj ;
    if (!EqualsUtil.equals(this.positivePaint, that.positivePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.negativePaint, that.negativePaint)) {
      return false ;
    }
    if (this.shapesVisible != that.shapesVisible) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.positivePaint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.negativePaint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.shapesVisible) ;

    return hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYDifferenceRender" ;
  }

  /**
   * <zh>л.</zh>
   * <en>Provides serialization support.</en>
   * @param stream <zh>.</zh>
   * <en>the output stream.</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writePaint(stream, this.positivePaint) ;
    SerialUtil.writePaint(stream, this.negativePaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.positivePaint = SerialUtil.readPaint(stream) ;
    this.negativePaint = SerialUtil.readPaint(stream) ;
  }
}
