package com.ijchart.xychart.render.xy ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>StepʽXYͼ.</zh>
 * <en>Draw xy steped area shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYStackedStepAreaRender extends AbstractXYAreaRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3414596793385691311L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYStackedStepAreaRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    checkRowDataDuplicate() ;
    Object[] newDataset = processDataset() ;

    int uniqueXCount = this.uniqueXValues.size() ;
    double[] xValuePosition = new double[uniqueXCount] ;
    for (int i = 0 ; i < uniqueXCount ; i++) { //xֵλ,
      Number xData = (Number)this.uniqueXValues.get(i) ;
      xValuePosition[i] = xAxis.valueToJava2D(dataArea, xAxisEdge, xData.doubleValue(), false) ;
    }

    double basePosition = yAxis.valueToJava2D(dataArea, yAxisEdge, 0, false) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    double[] allLowPos = new double[uniqueXCount * 3] ;
    double[] allHighPos = new double[uniqueXCount * 3] ;
    for (int i = 0 ; i < allLowPos.length ; i++) {
      allLowPos[i] = basePosition ;
      allHighPos[i] = basePosition ;
    }
    double[] posArr = new double[3] ;
    double leftCategoryWidth = 0 ;
    double rightCategoryWidth = 0 ;
    boolean isVertical = orientation.equals(ChartOrientation.VERTICAL) ;

    //ͼ
    for (int i = 0 ; i < rows ; i++) {
      arrToNull(posArr) ;
      double[][] currRowDouble = (double[][]) newDataset[i] ;
      for (int j = 0 ; j < uniqueXCount ; j++) {
        double shapeX = xValuePosition[j] ;
        leftCategoryWidth = 0 ;
        rightCategoryWidth = 0 ;
        if (j == 0) {
          if (j < uniqueXCount - 1) {
            rightCategoryWidth = (xValuePosition[j + 1] - xValuePosition[j]) * 0.5 ;
          }
        } else {
          leftCategoryWidth = (xValuePosition[j] - xValuePosition[j - 1]) * 0.5 ;
          if (j < uniqueXCount - 1) {
            rightCategoryWidth = (xValuePosition[j + 1] - xValuePosition[j]) * 0.5 ;
          }
        }

        double[] data = currRowDouble[j] ;
        if (data == null || Double.isNaN(data[0]) || Double.isNaN(data[1])) {
          arrToNull(posArr) ;
          renderShapes.addDataItemShape(null) ;
          continue ;
        }
        double currYPos = yAxis.valueToJava2D(dataArea, yAxisEdge, data[1], false) ;

        double nextYPos = Double.NaN ;
        if (j < uniqueXCount - 1) {
          double[] nextData = currRowDouble[j + 1] ;
          if (nextData != null) {
            double nextValue = nextData[1] ;
            if (!Double.isNaN(nextValue)) {
              nextYPos = yAxis.valueToJava2D(dataArea, yAxisEdge, nextValue, false) ;
            }
          }
        }

        //ֵ᷶Χֵ
        if (currYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
            || nextYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
          renderShapes.addDataItemShape(null) ;
          arrToNull(posArr) ;
          continue ;
        }

        boolean isCurrPositive = currYPos <= basePosition ;
        double realCurrPos = isCurrPositive ? allLowPos[j * 3 + 1] + currYPos - basePosition
                             : allHighPos[j * 3 + 1] + currYPos - basePosition ; //the real position of current data

        Rectangle2D drawArea = null ;
        GeneralPath path = new GeneralPath() ;
        if (isVertical) {
          if (j == 0) {
            path.moveTo((float) shapeX, (float) realCurrPos) ;
            posArr[0] = basePosition ;
            posArr[1] = realCurrPos ;
            posArr[2] = realCurrPos ;

            path.lineTo((float) (shapeX + rightCategoryWidth), (float) posArr[2]) ;
            path.lineTo((float) (shapeX + rightCategoryWidth),
                        (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;

            path.lineTo((float) shapeX,
                        (float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1])) ;
            path.closePath() ;
          } else {
            posArr[0] = posArr[1] = posArr[2] = realCurrPos ;
            path.moveTo((float) (shapeX - leftCategoryWidth), (float) posArr[0]) ;
            path.lineTo((float) shapeX, (float) posArr[1]) ;

            path.lineTo((float) (shapeX + rightCategoryWidth), (float) posArr[2]) ;
            path.lineTo((float) (shapeX + rightCategoryWidth),
                        (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;

            path.lineTo((float) shapeX,
                        (float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1])) ;
            path.lineTo((float) (shapeX - leftCategoryWidth),
                        (float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3])) ;
            path.closePath() ;
          }

          if (isCurrPositive) {
            drawArea = new Rectangle2D.Double(shapeX, posArr[1], 1,
                                              Math.abs(currYPos - basePosition)) ;
          } else {
            drawArea = new Rectangle2D.Double(shapeX, posArr[1] - Math.abs(currYPos - basePosition),
                                              1, Math.abs(currYPos - basePosition)) ;
          }
        } else {
          if (j == 0) {
            path.moveTo((float) realCurrPos, (float) shapeX) ;
            posArr[0] = basePosition ;
            posArr[1] = realCurrPos ;
            posArr[2] = realCurrPos ;

            path.lineTo((float) posArr[2], (float) (shapeX + rightCategoryWidth)) ;
            path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                        (float) (shapeX + rightCategoryWidth)) ;

            path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1]),
                        (float) shapeX) ;
            path.closePath() ;

          } else {
            posArr[0] = posArr[1] = posArr[2] = realCurrPos ;
            path.moveTo((float) posArr[0], (float) (shapeX - leftCategoryWidth)) ;

            path.lineTo((float) posArr[1], (float) shapeX) ;
            path.lineTo((float) posArr[2], (float) (shapeX + rightCategoryWidth)) ;
            path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                        (float) (shapeX + rightCategoryWidth)) ;

            path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1]),
                        (float) shapeX) ;
            path.lineTo((float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3]),
                        (float) (shapeX - leftCategoryWidth)) ;
            path.closePath() ;
          }

          if (isCurrPositive) {
            drawArea = new Rectangle2D.Double(posArr[1], shapeX,
                                              Math.abs(currYPos - basePosition), 1) ;
          } else {
            drawArea = new Rectangle2D.Double(posArr[1] - Math.abs(currYPos - basePosition),
                                              shapeX, Math.abs(currYPos - basePosition), 1) ;
          }
        }

        Object[] params = {path, drawArea} ;
        IChartShape shape = createItemShape(orientation, yAxis, i, (int) data[2], null,
                                            fillPaints[i], borderPaints[i], borderStrokes[i],
                                            alpha, params) ;
        renderShapes.addDataItemShape(shape) ;

        if (isCurrPositive) {
          allLowPos[j * 3] = posArr[0] ;
          allLowPos[j * 3 + 1] = posArr[1] ;
          allLowPos[j * 3 + 2] = posArr[2] ;
        } else {
          allHighPos[j * 3] = posArr[0] ;
          allHighPos[j * 3 + 1] = posArr[1] ;
          allHighPos[j * 3 + 2] = posArr[2] ;
        }
      }
    }

  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    Object[] params = (Object[]) appendParam ;
    GeneralPath path = (GeneralPath) params[0] ;
    Rectangle2D drawArea = (Rectangle2D) params[1] ;

    GeneralPathChartShape shape = new GeneralPathChartShape(path, drawArea, null) ;
    shape.setText(text) ;
    shape.setForceInsideTextNoWrap(true) ;

    return shape ;
  }

  /**
   * <zh>еֵΪDouble.NaN.</zh>
   * <en>.</en>
   * @param arr double[]
   */
  private static void arrToNull(double[] arr) {
    for (int i = 0 ; i < arr.length ; i++) {
      arr[i] = Double.NaN ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYStackedStepAreaRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYStackedStepAreaRender" ;
  }

}
