package com.ijchart.xychart.util ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ַĹ</zh>
 * <en>Nice static helpers for working with Strings</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class StringUtil {

  /**
   * <zh>ַstrǷΪnullַ.</zh>
   * <en>Test whether or not string is null or empty.</en>
   *
   * @param str String <zh>ҪԵַ.</zh>
   * <en>The test string.</en>
   * @return boolean <zh>ַstrΪnullַtrue,򷵻false.</zh>
   * <en>If string is null or empty return true,otherwise return false.</en>
   */
  public static boolean isEmpty(String str) {
    return str == null || str.length() == 0 ;
  }

  /**
   * <zh>ַstrǷΪnullַ{@link String#trim()}ؿ.</zh>
   * <en>Test whether or not string is null or {@link String#trim()} return empty.</en>
   *
   * @param str String <zh>ҪԵַ.</zh>
   * <en>The test string.</en>
   * @return boolean <zh>ַstrΪnullַ{@link String#trim()}ؿ򷵻true,
   * 򷵻false.</zh>
   * <en>If string is null or empty or {@link String#trim()} is empty return true,
   * otherwise return false.</en>
   */
  public static boolean isEmptyWithTrim(String str) {
    return str == null || str.length() == 0 || str.trim().length() == 0 ;
  }

  /**
   * <zh>ַstrΪnull,ؿַ,򷵻str.</zh>
   * <en>If str is null return empty string,otherwise return str.</en>
   *
   * @param str String <zh>string,(Ϊnull).</zh>
   * <en>A string (null permitted).</en>
   * @return String
   */
  public static String nullToEmpty(String str) {
    if (str == null) {
      return "" ;
    }

    return str ;
  }

  /**
   * <zh>ַstrΪnull,ؿַ͵"null",򷵻str.</zh>
   * <en>If str is null return "null",otherwise return str.</en>
   *
   * @param obj Object <zh>Object,(Ϊnull).</zh>
   * <en>A Object (null permitted).</en>
   *
   * @return String
   */
  public static Object nullToString(Object obj) {
    if (obj == null) {
      return "null" ;
    }

    return obj ;
  }

  /**
   * <zh>жеԪǷΪnull.</zh>
   * <en>Returns whether or not an array of elements are as null.</en>
   *
   * @param arr String[] <zh>ҪԵ (Ϊnull).</zh>
   * <en>An array (null permitted).</en>
   *
   * @param trim boolean <zh>Ƿȥո.</zh>
   * <en>Whether or not trim space.</en>
   *
   * @return boolean
   */
  public static boolean isEmpty(String[] arr, boolean trim) {
    if (arr == null || arr.length == 0) {
      return true ;
    }

    if (trim) {
      for (int i = 0 ; i < arr.length ; i++) {
        if (!isEmptyWithTrim(arr[i])) {
          return false ;
        }
      }
    } else {
      for (int i = 0 ; i < arr.length ; i++) {
        if (!isEmpty(arr[i])) {
          return false ;
        }
      }

    }

    return true ;
  }

  /**
   * <zh>һеֵз,ȥظ,ֻмֵ.</zh>
   * <en>Remove repeat values and reserve the center object.</en>
   *
   * @param values String[] <zh>ַ.</zh>
   * <en>The array of labels.</en>
   *
   * @return String[] <zh>,Ⱥ֮ǰһ.</zh>
   * <en>The result.</en>
   */
  public static String[] removeRepeatValues(String[] values) {
    if (values == null) {
      return null ;
    }

    String[] result = new String[values.length] ;
    for (int i = 0 ; i < values.length ; i++) {
      if (values[i] == null) {
        continue ;
      }

      int end = i ;
      for (int j = i + 1 ; j < values.length ; j++) {
        if (!values[i].equals(values[j])) {
          end = j - 1 ;
          break ;
        } else {
          end = j ;
        }
      }

      int index = (int) ((i + end) * 0.5) ;
      result[index] = values[index] ;
      i = end ;
    }

    return result ;
  }

  /**
   * Returns a string that is equivalent to the input string, but with
   * special characters converted to JavaScript escape sequences.
   *
   * @param input  the string to escape (<code>null</code> not permitted).
   *
   * @return A string with characters escaped.
   *
   * @since 1.0.13
   */
  /**
   * <zh>ַתΪjavascript﷨ַ.</zh>
   * <en>Returns a string that is equivalent to the input string, but with
   * special characters converted to JavaScript escape sequences.</en>
   *
   * @param str String <zh>Ҫתַ (Ϊnull).</zh>
   * <en>The input string (null permitted).</en>
   *
   * @return String <zh>תַ (Ϊnull).</zh>
   * <en>The result string (never null).</en>
   */
  public static String javascriptEscape(String str) {
    if (str == null) {
      return "" ;
    }

    StringBuffer result = new StringBuffer() ;
    int length = str.length() ;
    for (int i = 0 ; i < length ; i++) {
      char c = str.charAt(i) ;
      if (c == '\"') {
        result.append("\\\"") ;
      } else if (c == '\'') {
        result.append("\\'") ;
      } else if (c == '\\') {
        result.append("\\\\") ;
      } else if (c == '\r' || c == '\n') {
      } else {
        result.append(c) ;
      }
    }

    return result.toString() ;
  }

}
