package com.ijchart.xychart.util ;

import java.awt.Font ;
import java.awt.FontMetrics ;
import java.awt.Graphics ;
import java.awt.Toolkit ;
import java.awt.geom.Rectangle2D ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ַĹ</zh>
 * <en>Nice static helpers for working with Strings</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class TextUtil {

  /**
   * <zh>һַָͼλµı߽,ַа'\n'ỻ.</zh>
   * <en>Returns the bounds of the specified array of characters
   * in the specified Graphics context.If the string contains '\n' will be wrapped.</en>
   *
   * @param g2 Graphics <zh>ͼλ (Ϊnull).</zh>
   * <en>The specified Graphics context (null not permitted).</en>
   * @param str String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null permitted).</en>
   *
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringBound(Graphics g2, String str) {
    return getStringBound(g2, str, 0, (str == null) ? 0 : str.length()) ;
  }

  /**
   * <zh>һַָͼλµı߽,ַа'\n'ỻ.</zh>
   * <en>Returns the bounds of the specified array of characters
   * in the specified Graphics context.If the string contains '\n' will be wrapped.</en>
   *
   * @param g2 Graphics <zh>ͼλ (Ϊnull).</zh>
   * <en>The specified Graphics context (null not permitted).</en>
   * @param str String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null permitted).</en>
   * @param beginIndex int <zh>ַstrĿʼλ.</zh>
   * <en>The offset of the beginning of str</en>
   * @param endIndex int <zh>ַstrĽλ</zh>
   * <en>The offset of the ending of str</en>
   *
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringBound(Graphics g2, String str, int beginIndex, int endIndex) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (StringUtil.isEmpty(str)) {
      return new Rectangle2D.Double() ;
    }

    FontMetrics fm = g2.getFontMetrics() ;
    int descent = fm.getDescent() ;

    double maxWidth = 0 ;
    double totalHeight = 0 ;

    String substr = str.substring(beginIndex, endIndex) ;
    String[] strArr = substr.split("\n") ;
    for (int i = 0 ; i < strArr.length ; i++) {
      Rectangle2D bound = fm.getStringBounds(strArr[i], g2) ;
      maxWidth = Math.max(bound.getWidth() + 2, maxWidth) ;
      totalHeight += bound.getHeight() ;
    }
    totalHeight += descent * strArr.length ;

    return new Rectangle2D.Double(0, 0, maxWidth, totalHeight) ;
  }

  /**
   * <zh>һַָı߽.</zh>
   * <en>Returns the bounds of the specified array of characters in the specified font.</en>
   *
   * @param font Font <zh>.</zh>
   * <en>The font.</en>
   * @param str String <zh>ַָ.</zh>
   * <en>The specified String.</en>
   *
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringBound(Font font, String str) {
    if (font == null || StringUtil.isEmpty(str)) {
      return new Rectangle2D.Double() ;
    }

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font) ;
    String[] lines = str.split("\n") ;
    double maxWidth = 0.0, totalHeight = 0.0 ;
    for (int i = 0 ; i < lines.length ; i++) {
      maxWidth = Math.max(maxWidth, fm.stringWidth(lines[i])) ;
      totalHeight += fm.getHeight() ;
    }

    return new Rectangle2D.Double(0, 0, maxWidth, totalHeight) ;
  }

  /**
   * <zh>һַָͼλµı߽.</zh>
   * <en> Returns the bounds of the specified array of characters in the
   *  specified Graphics context.</en>
   *
   * @param g2 Graphics2D <zh>ͼλ.</zh>
   * <en>The specified Graphics context.</en>
   * @param str String <zh>ַָ.</zh>
   * <en>The specified String.</en>
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringBoundWithFm(Graphics g2, String str) {
    FontMetrics fm = g2.getFontMetrics() ;

    return new Rectangle2D.Double(0, -fm.getAscent() + fm.getDescent(), fm.stringWidth(str),
                                  fm.getHeight() - 2 * fm.getDescent() + 1) ;
  }

  /**
   * <zh>һַָͼλµĿ.</zh>
   * <en>Returns the width of the specified array of characters
   * in the specified Graphics context.</en>
   *
   * @param g2 Graphics <zh>ͼλ (Ϊnull).</zh>
   * <en>The specified Graphics context (null not permitted).</en>
   * @param str String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null permitted).</en>
   *
   * @return double <zh>ַ.</zh>
   * <en>The string width.</en>
   */
  public static double getStringWidth(Graphics g2, String str) {
    return getStringBound(g2, str, 0, str.length()).getWidth() ;
  }

  /**
   * <zh>һַָͼλµĸ߶.</zh>
   * <en>Returns the height of the specified array of characters
   * in the specified Graphics context.</en>
   *
   * @param g2 Graphics <zh>ͼλ (Ϊnull).</zh>
   * <en>The specified Graphics context (null not permitted).</en>
   * @param str String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null permitted).</en>
   *
   * @return double <zh>ַ߶.</zh>
   * <en>The string height.</en>
   */
  public static double getStringHeight(Graphics g2, String str) {
    return getStringBound(g2, str, 0, str.length()).getHeight() ;
  }

  /**
   * <zh>һַָͼλµı߽,ַа'\n'ỻ.</zh>
   * <en>Returns the bounds of the specified array of characters
   * in the specified Graphics context.If the string contains '\n' will be wrapped.</en>
   *
   * @param text String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null not permitted).</en>
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   *
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringWrapBounds(String text, Font font) {
    if (StringUtil.isEmpty(text)) {
      return new Rectangle2D.Double() ;
    }

    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    double maxWidth = 0 ;
    double totalHeight = 0 ;

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font) ;
    String[] strArr = text.split("\n") ;
    for (int i = 0 ; i < strArr.length ; i++) {
      double width = fm.stringWidth(strArr[i]) ;
      if (width > maxWidth) {
        maxWidth = width ;
      }
      totalHeight += fm.getHeight() ;
    }

    return new Rectangle2D.Double(0, 0, maxWidth, totalHeight) ;
  }

  /**
   * <zh>һַָͼλµı߽,ǻз'\n'.</zh>
   * <en>Returns the bounds of the specified array of characters
   * in the specified Graphics context.If the string contains '\n' will be not wrapped.</en>
   *
   * @param text String <zh>ַָ (Ϊnull).</zh>
   * <en>The specified String (null not permitted).</en>
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   *
   * @return Rectangle2D <zh>ַָ߽.</zh>
   * <en>A Rectangle2D that is the bounding box of the specified String in the specified.</en>
   */
  public static Rectangle2D getStringNoWrapBounds(String text, Font font) {
    if (StringUtil.isEmpty(text)) {
      return new Rectangle2D.Double() ;
    }

    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font) ;

    return new Rectangle2D.Double(0, 0, fm.stringWidth(text), fm.getHeight()) ;
  }
}
