package com.ijchart.xychart.web ;

import java.io.Serializable ;
import java.util.Collection ;
import java.util.Iterator ;

import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.AbstractPlot ;
import com.ijchart.xychart.render.AbstractRender ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>{@link IAreaFragment}ӿڵĳʵ.</zh>
 * <en>The default implement of interface {@link IAreaFragment}.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DefaultAreaFragment implements IAreaFragment, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1800676616032068102L ;

  /**
   * <zh>ȱʡareastyleֵ.</zh>
   * <en>The default style value.</en>
   */
  public static final String DEFAULT_STYLE = " style=\"cursor:hand\"" ;

  /**
   * <zh>ȱʡURLǰ׺.</zh>
   * <en>The default url prefix.</en>
   */
  public static final String DEFAULT_URL_PREFIX = "#" ;

  /**
   * <zh>ǷURL.</zh>
   * <en>A boolean flag controls whether or not generate URL.</en>
   */
  public static final boolean DEFAULT_GENERATE_URL = true ;

  /**
   * <zh>ȱʡclick.</zh>
   * <en>The default click function name.</en>
   */
  public static final String DEFAULT_CLICK_FUNCTION_NAME = "area_click" ;

  /**
   * <zh>IJChart.</zh>
   * <en>The IJChart object.</en>
   */
  private IJChart chart ;

  /**
   * <zh>areastyleֵ.</zh>
   * <en>The style value.</en>
   */
  private String style ;

  /**
   * <zh>ǷURL.</zh>
   * <en>A boolean flag controls whether or not generate URL.</en>
   */
  private boolean generateURL ;

  /**
   * <zh>URLǰ׺.</zh>
   * <en>The url prefix.</en>
   */
  private String URLPrefix ;

  /**
   * <zh>click.</zh>
   * <en>The click function name.</en>
   */
  private String clickFunctionName ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   */
  public DefaultAreaFragment(IJChart chart) {
    this(chart, DEFAULT_STYLE) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   * @param style String <zh>areastyleֵ (Ϊnull).</zh>
   * <en>The style value (null permitted).</en>
   */
  public DefaultAreaFragment(IJChart chart, String style) {
    if (chart == null) {
      throw new IllegalArgumentException("Null 'chart' argument.") ;
    }

    this.chart = chart ;
    this.style = style ;
    this.generateURL = DEFAULT_GENERATE_URL ;
    this.URLPrefix = DEFAULT_URL_PREFIX ;
    this.clickFunctionName = DEFAULT_CLICK_FUNCTION_NAME ;
  }

  /**
   * <zh>IJChart.</zh>
   * <en>Returns the IJChart object.</en>
   *
   * @return IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (never null).</en>
   */
  public IJChart getChart() {
    return this.chart ;
  }

  /**
   * <zh>IJChart.</zh>
   * <en>Sets the IJChart object.</en>
   *
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   */
  public void setChart(IJChart chart) {
    if (chart == null) {
      throw new IllegalArgumentException("Null 'chart' argument.") ;
    }

    this.chart = chart ;
  }

  /**
   * <zh>areastyleֵ.</zh>
   * <en>Returns the style value.</en>
   *
   * @return String <zh>(Ϊnull).</zh>
   * <en>(maybe null).</en>
   */
  public String getStyle() {
    return this.style ;
  }

  /**
   * <zh>areastyleֵ.</zh>
   * <en>Sets the style value.</en>
   *
   * @param style String <zh>(Ϊnull).</zh>
   * <en>(null permitted).</en>
   */
  public void setStyle(String style) {
    this.style = style ;
  }

  /**
   * <zh>ǷURL.</zh>
   * <en>Returns whether or not generate URL.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean getGenerateURL() {
    return this.generateURL ;
  }

  /**
   * <zh>ǷURL.</zh>
   * <en>Sets whether or not generate URL.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setGenerateURL(boolean flag) {
    this.generateURL = flag ;
  }

  /**
   * <zh>URLǰ׺.</zh>
   * <en>Returns the url prefix.</en>
   *
   * @return String <zh>URLǰ׺.</zh>
   * <en>The url prefix.</en>
   */
  public String getURLPrefix() {
    return this.URLPrefix ;
  }

  /**
   * <zh>URLǰ׺.</zh>
   * <en>Sets the url prefix.</en>
   *
   * @param url String <zh>URLǰ׺ (Ϊnull).</zh>
   * <en>The url prefix (null permitted).</en>
   */
  public void setURLPrefix(String url) {
    this.URLPrefix = url ;
  }

  /**
   * <zh>click.</zh>
   * <en>Returns the click function name.</en>
   *
   * @return String <zh>click (Ϊnull).</zh>
   * <en>The click function name (maybe null).</en>
   */
  public String getClickFunctionName() {
    return this.clickFunctionName ;
  }

  /**
   * <zh>click.</zh>
   * <en>Sets the click function name.</en>
   *
   * @param functionName String <zh>click (Ϊnull).</zh>
   * <en>The click function name (null permitted).</en>
   */
  public void setClickFunctionName(String functionName) {
    this.clickFunctionName = functionName ;
  }

  /**
   * <zh>ȵareaĲַ.</zh>
   * <en>Returns the area fragment of hotspot shape.</en>
   *
   * @param shapeIndex int <zh>ͼͬһͼμϵ.</zh>
   * <en>The index of shape in collection.</en>
   * @param data IChartData <zh>{@link IChartData} (Ϊnull).</zh>
   * <en>The {@link IChartData} object (maybe null).</en>
   *
   * @return String <zh>ַΪ<area>ǩһ.</zh>
   * <en>The area segment.</en>
   */
  public String getFragment(int shapeIndex, IChartData data) {
    String appendFragment = getAppendAreaFragment(shapeIndex, data) ;
    appendFragment = StringUtil.nullToEmpty(appendFragment) ;
    if (!appendFragment.startsWith(" ")) {
      appendFragment = " " + appendFragment ;
    }

    String clickFunction = "" ;
    if (!StringUtil.isEmpty(this.clickFunctionName)) {
      clickFunction = " onclick=\"return " + this.clickFunctionName + "(" + shapeIndex + ")\"" ;
    }

    if (data == null) {
      String urlStr = "" ;
      if (this.generateURL) {
        urlStr = " href=\"" + DEFAULT_URL_PREFIX + "\"" ;
      }

      return clickFunction + urlStr + StringUtil.nullToEmpty(this.style) + appendFragment ;
    }

    Object rowKey = null ;
    Object colKey = null ;
    boolean isFind = false ; //Ƿҵ

    AbstractPlot plot = this.chart.getPlot() ;
    Collection renders = plot.getRenders() ;
    Iterator iter = renders.iterator() ;
    while (iter.hasNext() && !isFind) { //пжݼ
      AbstractRender render = (AbstractRender) iter.next() ;
      IMatrixDataset dataset = render.getDataset() ;

      int rowCnt = dataset.getRowCount() ;
      int columnCnt = dataset.getColumnCount() ;
      for (int i = 0 ; i < rowCnt && !isFind ; i++) {
        for (int j = 0 ; j < columnCnt && !isFind ; j++) {
          IChartData otherData = dataset.getData(i, j) ;
          if (data == otherData) {
            rowKey = dataset.getRowKey(i) ;
            colKey = dataset.getColKey(j) ;
            isFind = true ;
          }
        }
      }
    }

    String urlStr = "" ;

    StringBuffer buffer = new StringBuffer() ;
    if (isFind) {
      String rowKeyStr = null ;
      if (rowKey != null) {
        rowKeyStr = rowKey.toString() ;
      }
      String colKeyStr = null ;
      if (colKey != null) {
        colKeyStr = colKey.toString() ;
      }

      boolean rowKeyNull = StringUtil.isEmpty(rowKeyStr) ;
      boolean colKeyNull = StringUtil.isEmpty(colKeyStr) ;
      String titleText = "" ;
      if (!rowKeyNull) {
        titleText += StringUtil.javascriptEscape(rowKeyStr) ;
      }
      if (!colKeyNull) {
        if (!rowKeyNull) {
          titleText += "," ;
        }
        titleText += StringUtil.javascriptEscape(colKeyStr) ;
      }
      if (!StringUtil.isEmpty(titleText)) {
        titleText = "(" + titleText + ")" ;
      }
      String labelText = data.getLabelText() ;
      if (!StringUtil.isEmpty(labelText)) {
        titleText += " = " + StringUtil.javascriptEscape(labelText) ;
      }

      buffer.append(" title=\"" + titleText + "\"") ;

      if (this.generateURL) {
        if (StringUtil.isEmpty(this.URLPrefix)) {
          urlStr = " href=\"" + DEFAULT_URL_PREFIX + "\"" ;
        } else {
          String url = this.URLPrefix ;
          if (!url.equals(DEFAULT_URL_PREFIX)) {
            int idx = url.indexOf("?") ;
            if (idx < 0) {
              url += "?" ;
            }
            url += "ROW_KEY=" + rowKeyStr + "&COLUMN_KEY=" + colKeyStr ;
            String dataUrl = data.getAreaURLFragment() ;
            if (dataUrl != null && !dataUrl.startsWith("&")) {
              url += "&" + dataUrl ;
            }
          }
          urlStr = " href=\"" + url + "\"" ;
        }
      }
    }

    buffer.append(clickFunction) ;
    buffer.append(urlStr) ;
    buffer.append(StringUtil.nullToEmpty(this.style)) ;
    buffer.append(appendFragment) ;

    return buffer.toString() ;
  }

  /**
   * <zh>õӵareaַ,ûʵ,Ҫַʵֱ.</zh>
   * <en>Returns the append area fragment.</en>
   *
   * @param shapeIndex int <zh>ͼͬһͼμϵ.</zh>
   * <en>The index of shape in collection.</en>
   * @param data IChartData <zh>{@link IChartData} (Ϊnull).</zh>
   * <en>The {@link IChartData} object (maybe null).</en>
   *
   * @return String
   */
  protected String getAppendAreaFragment(int shapeIndex, IChartData data) {
    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof DefaultAreaFragment)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    DefaultAreaFragment that = (DefaultAreaFragment) obj ;
    if (!EqualsUtil.equals(this.style, that.style)) {
      return false ;
    }
    if (this.generateURL != that.generateURL) {
      return false ;
    }
    if (!EqualsUtil.equals(this.URLPrefix, that.URLPrefix)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.chart, that.chart)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.chart) ;
    result = 37 * result + HashcodeUtil.hashCode(this.style) ;
    result = 37 * result + HashcodeUtil.hashCode(this.generateURL) ;
    result = 37 * result + HashcodeUtil.hashCode(this.URLPrefix) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
